/****************************************************************************
 *
 * @file	main.c
 * @brief	This is ct-sample program for S1C17656 demonstration.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 ***************************************************************************/
/****************************************************************************
 * @mainpage	S1C17656 CT
 *
 * @section		intro Introduction
 * Sample code occur CT interrupt, and calculate an elapsed time.
 * When wait T8 timer interrupt, the code changes Halt mode of CPU,
 * reduce consumption electricity.
 *
 * @subsection	step1	Step 1: Start CT.
 *						Start CT.
 * @subsection	step2	Step 2: Get T8 timer count data.
 *						When occur T8 timer interrupt, calculate an elapsed time
 *						since sample code start.
 *						Display an elapsed time to simulated I/O window.
 * @subsection	step3	Step 3: Repeat ten times.
 *						Repeat T8 timer interrupt ten times.
 * @subsection	step4	Step 4: Stop CT.
 *						Stop CT.
 *						End sample code.
 *
 * @section		install Installation
 * Start the IDE and import the "s1c17656_ct" project.
 *
 ***************************************************************************/

/// #include
#include <string.h>
#include <stdio.h>
#include <c17656_reg.h>
#include <init.h>
#include <clg.h>
#include <ct.h>

/// Prototype
int main(void);
void IntCT(void)__attribute__ ((interrupt_handler));

/// global
#define LOOPCOUNT_MAX		( 0x0a )		///< Loop count max value
#define CT_INT_XHZ			CT_INT_1HZ		///< Interrupt select

volatile unsigned int loopCount;
volatile unsigned int ctFlag = 0;

/****************************************************************************
 * Main function.
 *
 * @retval	1	Normality
 * @retval	0	Abnormality
 ***************************************************************************/
int main(void)
{
	int rnf;
	int rnf_main = 1;

	DebugModeMisc( 0 );			/// Misc Stop in Debug Mode
	DebugModePsc( 0 );			/// Prescaler Stop in Debug Mode
	SetClockGear( 0 );			///Set Clock Gear *default=0x0
	ControlClg( 3 );			/// Run Clg

	/// OSC1A Wait Cycle
	rnf = SetWaitCycleClg( 0, 0 );
	if ( rnf == 0 ) {
		printf("***  SetWaitCycleClg ERROR  ***\n");
		rnf_main = 0;
	}
	/// OSC1A run enable
	rnf = StartOSC1();
	if( rnf == 0 ){
		printf("***  StartOSC1 ERROR  ***\n");
		rnf_main = 0;
	}

	int		i,n;
	for( i = 0 ; i < 61; i++ ){			/// Wait 16384 * 2000000/32768 Cycle
		for( n = 0 ; n < 16384; n++ ){
			asm ( "nop" );
		}
	}

	/// Reset counter
	ResetCTCount();
	/// CT Initialize interrupt
	rnf = InitCTInt( 3 );
	if( rnf == 0 ){
		printf("***  InitCTInt ERROR  ***\n");
		rnf_main = 0;
	}
	/// CT Interrupt enable
	rnf = EnableCTInt( 0x01 );
	if( rnf == 0 ){
		printf("***  EnableCTInt ERROR  ***\n");
		rnf_main = 0;
	}

	printf( "<<< Clock Timer demonstration start >>>\n" );

	/// Start CT
	StartCT();

	loopCount = 1U;
	while ( loopCount <= LOOPCOUNT_MAX ) {
		asm( "halt" );
		asm( "nop" );

		if(ctFlag == 1) {
			printf("*** %01u.%01u sec ***\n", loopCount, (loopCount * 10U) % 10U);
			loopCount++;
		}
	}

	printf( "<<< CT  demonstration finish >>>\n" );

	/// Stop CT
	StopCT();
	/// Disable interrupt
	rnf = DisableCTInt(0x01);
	if( rnf == 0 ){
		printf("***  DisableCTInt ERROR  ***\n");
		rnf_main = 0;
	}

	return ( rnf_main );
}

/****************************************************************************
 * IntCT function.
 *
 * @brief	CT interrupt routine.
 ***************************************************************************/
void IntCT(void)
{
	if( isCTInt(0x01) ){

		int ret = 0;

		ctFlag = 1;
		/// Clear interrupt flag
		ret = ClrCTIntFlg( 0x01 );
		if( ret == 0 ){
			printf("***  ClrCTIntFlg ERROR  ***\n");
			ctFlag = 0;
		}
	}
}

