/****************************************************************************
 *
 * @file	lcd.c
 * @brief	This is lcd driver program file for S1C17656.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 ***************************************************************************/

#include <c17656_reg.h>
#include <lcd.h>

/*
 * definition of constants
 */
static const unsigned char	DOT_DATA[] = {
	0x01, 0x02, 0x04, 0x08
};

/****************************************************************************
 * InitLcdPower function.
 *
 * @brief	This function initializes LCD power.
 *
 * @param	selectBclkd
 *	@li @c		BCLKDIV_3B64		OSC3B:1/64,OSC3A:1/128
 *	@li @c		BCLKDIV_3B128		OSC3B:1/128,OSC3A:1/256
 *	@li @c		BCLKDIV_3B256		OSC3B:1/256,OSC3A:1/512
 *	@li @c		BCLKDIV_3B512		OSC3B:1/512,OSC3A:1/1024
 *	@li @c		BCLKDIV_3B1024		OSC3B:1/1024,OSC3A:1/2048
 *	@li @c		BCLKDIV_3B2048		OSC3B:1/2048,OSC3A:1/4096
 *	@li @c		BCLKDIV_3B4096		OSC3B:1/4096,OSC3A:1/8192
 *	@li @c		BCLKDIV_OSC1_8		OSC1:1/8
 *	@li @c		BCLKDIV_OSC1_16		OSC1:1/16
 *	@li @c		BCLKDIV_OSC1_32		OSC1:1/32
 *	@li @c		BCLKDIV_OSC1_64		OSC1:1/64
 * @param	selectBclksrc
 *	@li @c		LCD_CLKSRC_OSC3B	OSC3B
 *	@li @c		LCD_CLKSRC_OSC1		OSC1
 * @param	selectLcdbclke
 *	@li @c		LCDBCLKLCDTCLKE_DISABLE		LCD booster clock disable
 *	@li @c		LCDBCLKLCDTCLKE_ENABLE		LCD booster clock enable
 * @param	selectMode
 *	@li @c		LCDVREGLHVLD_OFF	Vc heavy load protection mode off
 *	@li @c		LCDVREGLHVLD_ON		Vc heavy load protection mode on
 * @param	selectVoltage
 *	@li @c		LCDVREGVCSEL_VC1	Vc1
 *	@li @c		LCDVREGVCSEL_VC2	Vc2
 *
 * @retval	LCD_SET_NG		Failed.
 * @retval	LCD_SET_OK		Success.
 ***************************************************************************/
int InitLCDPower( unsigned char selectBclkd, unsigned char selectBclksrc,
					unsigned char selectLcdbclke, unsigned char selectMode, unsigned char selectVoltage )
{
	/// Parameter Check
	if( (selectBclkd > 6)	||
		(selectBclksrc > 1)	||
		(selectLcdbclke > 1)||
		(selectMode > 1)	||
		(selectVoltage > 1)	){
			return 0;
	}
	if( selectBclksrc ==  1 ){
		if( selectBclkd > 3 ){
			return 0;
		}
	}

	LCD_BCLK = ((selectBclkd << 4) | (selectBclksrc << 2) | selectLcdbclke);
	LCD_VREG = ((selectMode << 4) | selectVoltage);

#if 0
	/// LCD Booster Clock Division Ratio Select
	LCD_BCLK_LCDBCLKD = selectBclkd;

	/// LCD Booster Clock Source Select
	LCD_BCLK_LCDBCLKSRC = selectBclksrc;

	/// LCD Booster Clock Enable Select
	LCD_BCLK_LCDBCLKE = selectLcdbclke;

	/// LCD Voltage Regulator Select
	LCD_VREG_LHVLD = selectMode;

	/// LCD Heavy Protection Mode Select
	LCD_VREG_VCSEL = selectVoltage;
#endif

	return 1;
}

/****************************************************************************
 * InitLcd function.
 *
 * @brief	This function initializes LCD.
 *
 * @param	selectlcdtclkd
 *	@li @c		LCD_CLKDIV_1024		1/1024
 *	@li @c		LCD_CLKDIV_2048		1/2048
 *	@li @c		LCD_CLKDIV_4096		1/4096
 *	@li @c		LCD_CLKDIV_8192		1/8192
 * @param	selectlcdtclk
 *	@li @c		LCD_CLKSRC_OSC3B	OSC3B
 *	@li @c		LCD_CLKSRC_OSC1		OSC1
 * @param	selectdsprev
 *	@li @c		LCDTDCTLDSPREV_REVERSE		Reverse
 *	@li @c		LCDTDCTLDSPREV_NORMAL		Normal
 * @param	selectdspc
 *	@li @c		LCD_DSPOFF		Display Off
 *	@li @c		LCD_NMRDSP		Normal Display
 *	@li @c		LCD_ALLON		All On
 *	@li @c		LCD_ALLOFF		All Off
 * @param	selectfrmcnt
 *	@li @c		LCD_FREQ_4		1/4
 *	@li @c		LCD_FREQ_8		1/8
 *	@li @c		LCD_FREQ_12		1/12
 *	@li @c		LCD_FREQ_16		1/16
 * @param	selectlduty
 *	@li @c		LCD_DUTY_STATIC		Static
 *	@li @c		LCD_DUTY_2			1/2
 *	@li @c		LCD_DUTY_3			1/3
 *	@li @c		LCD_DUTY_4			1/4
 *
 * @retval	LCD_SET_NG		Failed.
 * @retval	LCD_SET_OK		Success.
 ***************************************************************************/
int InitLCD( unsigned char selectlcdtclk, unsigned char selectlcdtclkd, unsigned char selectdsprev,
			 unsigned char selectdspc, unsigned char selectfrmcnt, unsigned char selectlduty )
{
	/// Parameter Check
	if( (selectlcdtclk > 1)	||
		(selectlcdtclkd > 3)||
		(selectdsprev > 1)	||
		(selectdspc > 3)	||
		(selectfrmcnt > 3)	||
		(selectlduty > 3)){
			return 0;
	}

	/// LCD Clock Disable
	StopLCDClock();

	/// LCD Clock Source Set
	LCD_TCLK = selectlcdtclk << 2;

	/// LCD Clock Division Ratio Set
	if( LCD_TCLK_LCDTCLKSRC != 1 ){
		LCD_TCLK = (LCD_TCLK & 0xcf) | (selectlcdtclkd << 4);
	}

	LCD_DCTL = ((selectdsprev << 4) | selectdspc);

	LCD_CCTL = ((selectfrmcnt << 6) | selectlduty);

#if 0
	/// LCD Reverse Control Set
	LCD_DCTL_DSPREV = selectdsprev;

	/// LCD Control Set
	LCD_DCTL_DSPC = selectdspc;

	/// LCD Frame Frequency Set
	LCD_CCTL_FRMCNT = selectfrmcnt;

	/// LCD Duty Set
	LCD_CCTL_LDUTY = selectlduty;
#endif

	return 1;
}

/****************************************************************************
 * SetLCDDisplay1Seg function.
 *
 * @brief	This function display 1 dot of LCD.
 *
 * @param	*seg
 *	@li @c		LCD_SEG00	SEG00
 *	@li @c		LCD_SEG01	SEG01
 *	@li @c		LCD_SEG02	SEG02
 *	@li @c		LCD_SEG03	SEG03
 *	@li @c		LCD_SEG04	SEG04
 *	@li @c		LCD_SEG05	SEG05
 *	@li @c		LCD_SEG06	SEG06
 *	@li @c		LCD_SEG07	SEG07
 *	@li @c		LCD_SEG08	SEG08
 *	@li @c		LCD_SEG09	SEG09
 *	@li @c		LCD_SEG10	SEG10
 *	@li @c		LCD_SEG11	SEG11
 *	@li @c		LCD_SEG12	SEG12
 *	@li @c		LCD_SEG13	SEG13
 *	@li @c		LCD_SEG14	SEG14
 *	@li @c		LCD_SEG15	SEG15
 *	@li @c		LCD_SEG16	SEG16
 *	@li @c		LCD_SEG17	SEG17
 *	@li @c		LCD_SEG18	SEG18
 *	@li @c		LCD_SEG19	SEG19
 *	@li @c		LCD_SEG20	SEG20
 *	@li @c		LCD_SEG21	SEG21
 *	@li @c		LCD_SEG22	SEG22
 *	@li @c		LCD_SEG23	SEG23
 *	@li @c		LCD_SEG24	SEG24
 *	@li @c		LCD_SEG25	SEG25
 *	@li @c		LCD_SEG26	SEG26
 *	@li @c		LCD_SEG27	SEG27
 *	@li @c		LCD_SEG28	SEG28
 *	@li @c		LCD_SEG29	SEG29
 *	@li @c		LCD_SEG30	SEG30
 *	@li @c		LCD_SEG31	SEG31
 * @param	com
 *	@li @c		LCD_COM00	COM00
 *	@li @c		LCD_COM01	COM01
 *	@li @c		LCD_COM02	COM02
 *	@li @c		LCD_COM03	COM03
 * @param	data
 *	@li @c		LCD_DATA_TURNOFF	LCD Data Turn off
 *	@li @c		LCD_DATA_TURNON		LCD Data Turn on
 *
 * @retval	LCD_SET_NG		Failed.
 * @retval	LCD_SET_OK		Success.
 ***************************************************************************/
int SetLCDDisplay1Seg( unsigned char* seg, unsigned char com, unsigned char data )
{
	unsigned char *lcdram;

	/// Parameter Check
	if( (*seg >= 32)	||
		(com >= 4)		||
		(data > 1)	){
			return 0;
	}

	if(data == 1){

		unsigned char num;

		for( lcdram = (unsigned char*)0x53c0 ; lcdram <= (0x53c0 + seg) ; lcdram++){
			for(num = 0; num <= com; num++){
				*lcdram |= data * DOT_DATA[num];
			}
		}
	} else {
		for( lcdram = (unsigned char*)0x53c0 ; lcdram <= (0x53c0 + seg) ; lcdram++){
			*lcdram = data;
		}
	}

	return 1;
}

/****************************************************************************
 * StartLCDClock function.
 *
 * @brief	This function sets LCD clock enable.
 ***************************************************************************/
void StartLCDClock( void )
{
	/// LCD Clock Enable
	LCD_TCLK |= 0x01;
}

/****************************************************************************
 * StopLCDClock function.
 *
 * @brief	This function sets LCD clock disable.
 ***************************************************************************/
void StopLCDClock( void )
{
	/// LCD Clock Disable
	LCD_TCLK &= 0xfe;
}

/****************************************************************************
 * InitLCDInt function.
 *
 * @brief	This function initializes LCD interrupt.
 *
 * @param	intLevel
 *	@li @c		LCD_INTLV_0		Interrupt Level 0
 *	@li @c		LCD_INTLV_1		Interrupt Level 1
 *	@li @c		LCD_INTLV_2		Interrupt Level 2
 *	@li @c		LCD_INTLV_3		Interrupt Level 3
 *	@li @c		LCD_INTLV_4		Interrupt Level 4
 *	@li @c		LCD_INTLV_5		Interrupt Level 5
 *	@li @c		LCD_INTLV_6		Interrupt Level 6
 *	@li @c		LCD_INTLV_7		Interrupt Level 7
 *
 * @retval	CLG_STOP_NG		Failed.
 * @retval	CLG_STOP_OK		Success.
 ***************************************************************************/
short InitLCDInt( unsigned char intLevel )
{
	/// Parameter Check
	if( intLevel > 7 ){
			return 0;
	}

	/// LCD Interrupt Level Set
	ITC_LV3 = (ITC_LV3 & 0x0700 ) | intLevel;

	return 1;
}

/****************************************************************************
 * EnableLCDInt function.
 *
 * @brief	LCD interrupt enable.
 ***************************************************************************/
void EnableLCDInt( void )
{
	/// Clear LCD interrupt
	ClrLCDIntFlg();

	/// LCD interrupt enable
	LCD_IMSK |= 0x01;
}

/****************************************************************************
 * DisableLCDInt function.
 *
 * @brief	LCD interrupt disable.
 ***************************************************************************/
void DisableLCDInt( void )
{
	/// LCD interrupt enable
	LCD_IMSK &= 0xfe;

	/// Clear LCD interrupt
	ClrLCDIntFlg();
}


/****************************************************************************
 * isLCDInt function.
 *
 * @brief	This function check interrupt flag.
 *
 * @retval	LCDIFLGIFRMLG_NOTOCCURED	Not occurred
 * @retval	LCDIFLGIFRMLG_OCCURRED		Occurred.
 ***************************************************************************/
int isLCDInt(void)
{
	/// Interrupt Check
	if( LCD_IFLG_IFRMFLG != 1 ){
		return 0;
	}

	/// Interrupt Flag Reset
	LCD_IFLG = 0x01;

	return 1;
}

/****************************************************************************
 * ClrLCDIntFlg function.
 *
 * @brief	Clear LCD interrupt flag
 ***************************************************************************/
void ClrLCDIntFlg( void )
{
	/// Clear LCD interrupt
	LCD_IFLG = 0x01;
}

