/****************************************************************************
 *
 * @file	init.c
 * @brief	This is initialize program for S1C17656 demonstration.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 ***************************************************************************/

/* #include */
#include <c17656_reg.h>
#include <init.h>

/****************************************************************************
 * DebugModePsc
 *
 * @brief	Prescaler run/stop setting in debug mode.
 *
 * @param	debugMode
 *	@li @c		0	Stop
 *	@li @c		1	Run
 ***************************************************************************/
void DebugModePsc(unsigned char debugMode)
{
	/// Prescaler run/stop setting in debug mode
	MISC_DMODE1 = (debugMode << 1);
}

/****************************************************************************
 * ControlClg
 *
 * @brief	Clg run/stop control.
 *
 * @param	control
 *	@li @c		0	Stop
 *	@li @c		1	Run
 ***************************************************************************/
void ControlClg(unsigned char control)
{
	/// PCLK enable
	CLG_PCLK = control;
}

/****************************************************************************
 * SetClockGear
 *
 * @brief	Clock Gear setting.
 *
 * @param	gear	1/x
 ***************************************************************************/
void SetClockGear(unsigned char gear)
{
	/// Clock Gear ratio select
	CLG_CCLK = gear;
}

/****************************************************************************
 * SetFlashcAccessCycle
 *
 * @brief	Flash access cycle setting.
 *
 * @param	flashc	flashc
 ***************************************************************************/
void SetFlashcAccessCycle(unsigned char flashc)
{

	ProtectMisc( 0x96 );
	/// Flashc read access cycle
	FLASHC_WAIT = flashc;
	ProtectMisc( 0x00 );

}

/****************************************************************************
 * DebugModeMisc
 *
 * @brief	OSC1 peripheral control in Debug mode.
 *
 * @param	debugMode
 *	@li @c		0	Stop
 *	@li @c		1	Run
 ***************************************************************************/
void DebugModeMisc(unsigned char debugMode)
{
	/// OSC1 peripheral run/stop control in Debug mode
	MISC_DMODE2 = debugMode;
}

/****************************************************************************
 * ProtectMisc
 *
 * @brief	Misc register write protect mode setting.
 *
 * @param	protectCode
 *	@li @c		0x96		remove the write protect
 *	@li @c		another		write protect
 ***************************************************************************/
void ProtectMisc(unsigned char protectCode)
{
	/// Misc register write protect
	MISC_PROT = protectCode;
}

/****************************************************************************
 * SetMiscIramSize
 *
 * @brief	Iram size setting.
 *
 * @param	iramSize
 *	@li @c		0x2		4KB
 *	@li @c		0x3		2KB
 ***************************************************************************/
void SetMiscIramSize(unsigned char iramSize)
{
	ProtectMisc( 0x96 );
	/// Iram size select
	MISC_IRAMSZ_IRAMSZ = iramSize;
	ProtectMisc( 0x00 );
}

/****************************************************************************
 * SetMiscVecAddress
 *
 * @brief	Vector table address setting.
 *
 * @param	baseAdrL	base address[15:8]
 * @param	baseAdrH	base address[23:16]
 ***************************************************************************/
void SetMiscVecAddress(unsigned long baseAdrr)
{
	ProtectMisc( 0x96 );

	// Vector table base address low
	// Vector table base address high
	MISC_TTBR = baseAdrr;

	ProtectMisc( 0x00 );
}

