/****************************************************************************
 *
 * @file	main.c
 * @brief	This is clg(osc)-sample program for S1C17656 demonstration.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 ***************************************************************************/
/****************************************************************************
 * @mainpage	S1C17656 Oscillator Circuit (OSC)
 *
 * @section		intro Introduction
 * Sample code to switch the Oscillator (OSC) circuits.
 * The code switches OSC1 and OSC3B, and displays 1, 2, 3c to the
 * simulated I/O window at constant intervals.
 *
 * @subsection	step1	Step 1: Start OSC3B.
 *						Starts OSC3B.
 * @subsection	step2	Step 2: Change from OSC3B to OSC1.
 * 						Starts the OSC1.
 *						Switches the system clock from OSC3B to OSC1.
 *						Stops the OSC3B.
 * @subsection	step3	Step 3: Change from OSC1 to OSC3B.
 *						Starts the OSC3B.
 *						Switches the system clock from OSC1 to OSC3B.
 *						Stops the OSC1.
 *
 * @section		install Installation
 * Start the IDE and import the "s1c17656_osc_clg" project.
 *
 ***************************************************************************/

/// #include
#include <string.h>
#include <stdio.h>
#include <c17656_reg.h>
#include <init.h>
#include <clg.h>

/// Prototype
int main(void);
void softwareWait(void);

/****************************************************************************
 * Main function.
 *
 * @retval	1	Normality
 * @retval	0	Abnormality
 ***************************************************************************/
int main(void)
{
	int	i;
	int rnf;
	int rnf_main = 1;

	DebugModeMisc( 0 );			/// Misc Stop in Debug Mode
	DebugModePsc( 0 );			/// Prescaler Stop in Debug Mode
	SetClockGear( 0 );			/// Set Clock Gear *default=0x0
	ControlClg( 3 );			/// Run Clg

	/// OSC3B Wait Cycle
	rnf = SetWaitCycleClg( 3, 0 );
	if ( rnf == 0 ) {
		printf("***  SetWaitCycleClg ERROR  ***\n");
		rnf_main = 0;
	}

	/// Start OSC3B
	/// OSC3B do not Start, because works in division ratio of Default.

	printf( "<<< CLG(OSC) demonstration start >>>\n" );

	/// OSC3B
	for ( i = 1; i < 10; i++ )
	{
		softwareWait();
		printf( "OSC3B *** %d ***\n", i );
	}


	printf( "*** Change from OSC3B to OSC1 ***\n" );

	/// OSC1A Wait Cycle
	rnf = SetWaitCycleClg( 0, 0 );
	if ( rnf == 0 ) {
		printf("***  SetWaitCycleClg ERROR  ***\n");
		rnf_main = 0;
	}

	/// Start OSC1
	rnf = StartOSC1();
	if ( rnf == 0 ) {
		printf("***  StartOSC1 ERROR  ***\n");
		rnf_main = 0;
	}

	int n;

	for( i = 0 ; i < 61; i++ ){			/// Wait 16384 * 2000000/32768 Cycle
		for( n = 0 ; n < 16384; n++ ){
			asm ( "nop" );
		}
	}

	/// Change to OSC1A
	rnf = ChgOSC( 2 );
	if ( rnf == 0 ) {
		printf("***  ChgOSC ERROR  ***\n");
		rnf_main = 0;
	}
	/// Stop OSC3B
	rnf = StopOSC3B();
	if ( rnf == 0 ) {
		printf("***  StopOSC3B ERROR  ***\n");
		rnf_main = 0;
	}

	/// OSC1
	for ( i = 1; i < 10; i++ )
	{
		softwareWait();
		printf( "OSC1 *** %d ***\n", i );
	}

	printf( "*** Change from OSC1 to OSC3B ***\n" );


	/// OSC3B Wait Cycle
	rnf = SetWaitCycleClg( 3, 0 );
	if ( rnf == 0 ) {
		printf("***  SetWaitCycleClg ERROR  ***\n");
		rnf_main = 0;
	}
	/// Start OSC3B
	rnf = StartOSC3B( 0 );
	if ( rnf == 0 ) {
		printf("***  StartOSC3B ERROR  ***\n");
		rnf_main = 0;
	}

	asm ( "nop" );			/// Wait 64 * 32768/2000000 Cycle

	/// Change to OSC3B
	rnf = ChgOSC( 3 );
	if ( rnf == 0 ) {
		printf("***  ChgOSC ERROR  ***\n");
		rnf_main = 0;
	}
	/// Stop OSC1
	rnf = StopOSC1();
	if ( rnf == 0 ) {
		printf("***  StopOSC1 ERROR  ***\n");
		rnf_main = 0;
	}

	/// OSC3B
	for ( i = 1; i < 10; i++ )
	{
		softwareWait();
		printf( "OSC3B *** %d ***\n", i );
	}


	printf( "<<< CLG(OSC) demonstration finish >>>\n");

	return ( rnf_main );
}

/****************************************************************************
 * SoftwareWait function.
 *
 * @brief	Software wait.
 ***************************************************************************/
void softwareWait(void)
{
	int	i;
	for ( i=0; i<0x100; i++ ){
		;
	}
}
