/****************************************************************************
 *
 * @file	main.c
 * @brief	This is port-sample program for S1C17656 demonstration.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 ***************************************************************************/
/****************************************************************************
 * @mainpage	S1C17656 PORT
 *
 * @section		intro Introduction
 * Sample code detect input signal changed in LOW level by PORT input interrupt.
 * The code output HIGH or Low level signal by PORT output.
 *
 * @subsection	step1	Step 1: P00 interrupt.
 *						Change input signal of P00 PORT into LOW level.
 *						Display "P00 Interrupt" to simulated I/O window.
 *						Reverse P04 PORT output.
 * @subsection	step2	Step 2: P01 interrupt.
 *						Change input signal of P01 PORT into LOW level.
 *						Display "P01 Interrupt" to simulated I/O window.
 *						Reverse P05 PORT output.
 * @subsection	step3	Step 3: P02 interrupt.
 *						Change input signal of P02 PORT into LOW level.
 *						Display "P02 Interrupt" to simulated I/O window.
 *						Reverse P06 PORT output.
 * @subsection	step4	Step 4: P03 interrupt.
 *						Change input signal of P03 PORT into LOW level.
 *						Display "P03 Interrupt" to simulated I/O window.
 *						Reverse P07 PORT output.
 *
 * @section		install Installation
 * Start the IDE and import the "s1c17656_port" project.
 *
 ***************************************************************************/
/// #include
#include <string.h>
#include <stdio.h>
#include <c17656_reg.h>
#include <init.h>
#include <port.h>

/// Prototype
int main(void);
void IntP0x(void)__attribute__ ((interrupt_handler));

/// global
volatile unsigned char sw0Flag;
volatile unsigned char sw1Flag;
volatile unsigned char sw2Flag;
volatile unsigned char sw3Flag;

/****************************************************************************
 * Main function.
 *
 * @retval	1	Normality
 * @retval	0	Abnormality
 ***************************************************************************/
int main(void)
{
	unsigned char flgP0xOut;
	int rnf;
	int rnf_main = 1;

	DebugModeMisc( 0 );			/// Misc Stop in Debug Mode
	DebugModePsc( 0 );			/// Prescaler Stop in Debug Mode
	SetClockGear( 0 );			/// Set Clock Gear *default=0x0
	ControlClg( 3 );			/// Run Clg

	/// Initialize port interrupt flag
	sw0Flag = 0;
	sw1Flag = 0;
    sw2Flag = 0;
    sw3Flag = 0;

	flgP0xOut = 0;

	printf( "<<< Port demonstration start>>>\n" );

	/// P0 interrupt all disable
	DisableP0Int(0xFF);

	/// Chattering Filter Setup
	SetP0Chat( 7, 7 );

	/// Input Enable Setup
	SetPortInput( 0x0F, 0 ,0);

	/// Output Enable Setup
	SetPortOutput( 0xF0, 0 ,0);

	/// Output Data Setup
	rnf = SetPortOutputData(0, 0);
	if( rnf == PORT_SET_NG ){
		printf("***  SetPortOutputData ERROR  ***\n");
		rnf_main = 0;
	}

	/// Port0 Interrupt Setup
	rnf = InitP0Int( 3, 0x0F);
	if( rnf == PORT_SET_NG ){
		printf("***  InitP0Int ERROR  ***\n");

	}
	/// Port0 Interrupt enable
	EnableP0Int(0x0F);

	while ( 1 )
	{
		if ( sw0Flag ){
			sw0Flag = 0;
			printf( "** P00 Interrupt ***\n" );
			/// Reverse Port Output Data
			flgP0xOut ^= 0x10;
			rnf = SetPortOutputData( 0, flgP0xOut );
			if( rnf == PORT_SET_NG ){
				printf("***  SetPortOutputData ERROR  ***\n");
				rnf_main = 0;
			}
		}

		if ( sw1Flag ){
			sw1Flag = 0;
			printf( "** P01 Interrupt ***\n" );
			/// Reverse Port Output Data
			flgP0xOut ^= 0x20;
			rnf = SetPortOutputData( 0, flgP0xOut );
			if( rnf == PORT_SET_NG ){
				printf("***  SetPortOutputData ERROR  ***\n");
				rnf_main = 0;
			}
		}

		if ( sw2Flag ){
			sw2Flag = 0;
			printf( "** P02 Interrupt ***\n" );
			/// Reverse Port Output Data
			flgP0xOut ^= 0x40;
			rnf = SetPortOutputData( 0, flgP0xOut );
			if( rnf == PORT_SET_NG ){
				printf("***  SetPortOutputData ERROR  ***\n");
				rnf_main = 0;
			}
		}

		if ( sw3Flag ){
			sw3Flag = 0;
			printf( "** P03 Interrupt ***\n" );
			/// Reverse Port Output Data
			flgP0xOut ^= 0x80;
			rnf = SetPortOutputData( 0, flgP0xOut );
			if( rnf == PORT_SET_NG ){
				printf("***  SetPortOutputData ERROR  ***\n");
				rnf_main = 0;
			}
			/// Loop end
			break;
		}
	}

	printf( "<<< Port demonstration finish>>>\n" );

	/// Port0 interrupt disable
	DisableP0Int(0xFF);

	return ( rnf_main );
}

/****************************************************************************
 * IntP0x function.
 *
 * @brief	P0x interrupt routine.
 ***************************************************************************/
void IntP0x(void)
{
	if ( isP0Int(0x01) ) {
		sw0Flag = 1;
	} else if ( isP0Int(0x02) ) {
		sw1Flag = 1;
	} else if ( isP0Int(0x04) ) {
		sw2Flag = 1;
	} else if ( isP0Int(0x08) ) {
		sw3Flag = 1;
	} else {
		;
	}
}


