/******************************************************************************
 *
 * @file	rfc.c
 * @brief	This is R/F Converter program for S1C17656 demonstration.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 *****************************************************************************/

/* #include */
#include <c17656_reg.h>
#include <rfc.h>


/***************************************************************************
 * initRfc function.
 *
 * @brief	First, Stop RFC.
 *			Next, Initialize RFC.
 *
 * @retval	STATUS_OK	Success.
 * @retval	STATUS_NG	Failed.
 **************************************************************************/
int initRfc( void )
{
	/// When OSC1 doesn't start, nothing is done.
	if ( CLG_CTL_OSC1EN == 0 ) {
		return 0;
	}

	/// Stop RFC.
	stopRfc();

	/// Set clock source for RFC.
	RFC_CLK = 0x05;

//	RFC_CLK_CLKSRC		= 1;	/// - Set clock source(OSC1).
//	RFC_CLK_CLKDIV		= 0;	/// - Set division ratio for clock source(1/1).

	/// Assign the RFC RFIN/REF/SENA function to the ports.
	/// - P23 : RFC(RFCLKO).
	/// - P16 : RFC(SENA0).
	/// - P15 : RFC(REF0).
	/// - P14 : RFC(RFIN0).
	P14_17PMUX		= 0x15;
	P23PMUX			= 0x40;

	/// Disable continuous oscillation.
	/// Set external clock input mode.
	RFC_CTL = 0;

	return 1;
}


/***************************************************************************
 * startRfc function.
 *
 * @brief	Start RFC.
 **************************************************************************/
void startRfc( void )
{
	/// Clear interrupt flag.
	//RFC_IFLG_OVTCIF = 1;
	//RFC_IFLG_OVMCIF = 1;
	//RFC_IFLG_ESENBIF = 1;
	//RFC_IFLG_ESENAIF = 1;
	//RFC_IFLG_EREFIF = 1;
	RFC_IFLG = 0x001F;

	/// Disable interrupt.
	//RFC_IFLG_OVTCIE = 0;
	//RFC_IFLG_OVMCIE = 0;
	//RFC_IFLG_ESENBIE = 0;
	//RFC_IFLG_ESENAIE = 0;
	//RFC_IFLG_EREFIE = 0;
	RFC_IMSK = 0x0000;

	/// Start RFC.
	RFC_CTL	|= 0x0001;
}


/***************************************************************************
 * stopRfc function.
 *
 * @brief	Stop RFC.
 **************************************************************************/
void stopRfc( void )
{
	/// Stop RFC.
	RFC_CTL	&= 0xfffe;

	/// Clear interrupt flag.
	//RFC_IFLG_OVTCIF = 1;
	//RFC_IFLG_OVMCIF = 1;
	//RFC_IFLG_ESENBIF = 1;
	//RFC_IFLG_ESENAIF = 1;
	//RFC_IFLG_EREFIF = 1;
	RFC_IFLG = 0x001F;

	/// Disable interrupt.
	//RFC_IFLG_OVTCIE = 0;
	//RFC_IFLG_OVMCIE = 0;
	//RFC_IFLG_ESENBIE = 0;
	//RFC_IFLG_ESENAIE = 0;
	//RFC_IFLG_EREFIE = 0;
	RFC_IMSK = 0x0000;

}


/***************************************************************************
 * setRfcMeasurementCounter function.
 *
 * @brief	Set measurement counter data.
 *
 * @param	count		Measurement counter data.
 **************************************************************************/
void setRfcMeasurementCounter( unsigned long count )
{
	/// Set measurement counter data.
	RFC_MC = count;

}


/***************************************************************************
 * getRfcMeasurementCounter function.
 *
 * @brief	Get measurement counter data.
 *
 * @return	unsigned long		Measurement counter data.
 **************************************************************************/
unsigned long getRfcMeasurementCounter( void )
{
	unsigned long result = 0;

	/// Get measurement counter data.
	result = RFC_MC;

	return result;
}


/***************************************************************************
 * setRfcTimeBaseCounter function.
 *
 * @brief	Set Time base counter data.
 *
 * @param	count		Time base counter data.
 **************************************************************************/
void setRfcTimeBaseCounter( unsigned long count )
{
	/// Set Time base counter data.
	RFC_TC = count;

}


/***************************************************************************
 * getRfcTimeBaseCounter function.
 *
 * @brief	Get Time base counter data.
 *
 * @return	unsigned long		Time base counter data.
 **************************************************************************/
unsigned long getRfcTimeBaseCounter( void )
{
	unsigned long result = 0;

	/// Get Time base counter data.
	result = RFC_TC;

	return result;
}


/***************************************************************************
 * runRfcConvertingOperation function.
 *
 * @brief	RFC Converting Operations.
 *
 * @param	startCount		Start measurement count.
 * @param	sensorCount		Pointer variable that stores measurement counter.
 *
 * @retval	RFC_STS_CONV_SEN_A		Sensor A oscillation completion interrupt.
 * @retval	RFC_STS_CONV_ERR_OVTC	Time base counter overflow error.
 * @retval	RFC_STS_CONV_ERR_OVMC	Measurement counter overflow error.
 **************************************************************************/
int runRfcConvertingOperation( unsigned long startCount, unsigned long* sensorCount )
{
	int result = 0x8002;
	/// Set measurement counter data.
	setRfcMeasurementCounter( startCount );
	/// Set Time base counter data.
	setRfcTimeBaseCounter( 0x00000000 );

	/// Clear interrupt flag.
	//RFC_IFLG_OVTCIF     = 1;
	//RFC_IFLG_EREFIF     = 1;
	RFC_IFLG			= 0x0011;

	/// Start CR oscillation.
	//RFC_TRG_SREF		= 1;
	RFC_TRG				|= 0x0001;
	while ( RFC_IFLG_EREFIF == 0 ) {
		/// Time base counter overflow error.
		if ( RFC_IFLG_OVTCIF == 1 ) {
			//RFC0INTF_OVTCIF = 1;
			RFC_IFLG = 0x0010;
			return 0x8001;
		}
	}

	/// Clear interrupt flag.
	//RFC_IFLG_OVMCIF     = 1;
	//RFC_IFLG_ESENAIF    = 1;
	RFC_IFLG			= 0x000A;

	/// Start Sensor A oscillation.
	//RFC_TRG_SSENA		= 1;
	RFC_TRG				|= 0x0002;


	/// Wait interrupts.
	while ( 1 ) {
		/// Sensor A oscillation completion interrupt.
		if ( RFC_IFLG_ESENAIF == 1 ) {
			//RFC_IFLG_ESENAIF = 1;
			RFC_IFLG = 0x0002;
			*sensorCount = getRfcMeasurementCounter();
			result = 0x0001;
			break;
		}
		/// Measurement counter overflow error.
		if ( RFC_IFLG_OVMCIF == 1 ) {
			//RFC_IFLG_OVMCIF = 1;
			RFC_IFLG = 0x0008;
			result = 0x8002;
			break;
		}
	}

	return result;
}

