/****************************************************************************
 *
 * @file	main.c
 * @brief	This is rtc-sample program for S1C17656 demonstration.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 ***************************************************************************/
/****************************************************************************
 * @mainpage	S1C17656 Real Time Clock (RTC)
 *
 * @section		intro Introduction
 * Sample code for generating interruption at intervals of 1Hz after setting time to RTC,
 * and displays 1, 2, 3c to the simulated I/O window.
 * @subsection	step1	Step 1: Start OSC1.
 *						Starts the OSC1.
 * @subsection	step2	Step 2: Start RTC.
 * 						RTC setting information is decided on Simulated I/O.
 *						Starts the RTC.
 * @subsection	step3	Step 3: Wait RTC 1Hz Interrupt.
 *						It waits until 1HZ interrupt occurs 10 times.
 * @subsection	step4	Step 4: Stop RTC.
 *						Stops the RTC.
 *
 * @section		install Installation
 * Start the IDE and import the "s1c17656_rtc" project.
 *
 ***************************************************************************/

/// #include
#include <string.h>
#include <stdio.h>
#include <c17656_reg.h>
#include <init.h>
#include <rtc.h>

#define	WAIT_CYCLE_00		(3)
#define	SELECT_OSC1B		(1)
#define	OSC1EN_ON			(1)
#define	WAIT_8Cycle			(8)
#define	RTC_INTERRUPT_CNT_MAX	(10)
#define	PREFIX_LENGTH		(2)
#define	INTERRUPT_MESSAGE		"> interrupt count value = "
#define	INTERRUPT_DISP_BUF		50

/// Prototype
int main(void);
static int setRTC(void);
static	void	CtoStr(char *cRes,int iChgVal);
void IntRtc(void)__attribute__ ((interrupt_handler));

/// global
volatile int rtcIntCt;	///RTC Interrupt Counter

/****************************************************************************
 * Main function.
 *
 * @brief	First,A MISC register is set up.
 *			Second,OSC1B is started.
 *			Third,A setRTC function is called,
 *			RTC information is set up and RTC is started.
 * 			Next,It waits until 1HZ interrupt occurs 10 times.
 *			then,RTC is stopped.
 *
 * @retval	RTC_SET_OK	Success.
 * @retval	RTC_SET_NG	Failed.
 ***************************************************************************/
int main(void)
{
	int		i,n;
	int		iRet = 1;
	int		rnf_main = 1;

	DebugModeMisc( 0 );			/// Misc Stop in Debug Mode
	DebugModePsc( 0 );			/// Prescaler Stop in Debug Mode
	SetClockGear( 0 );			/// Set Clock Gear *default=0x0
	ControlClg( 3 );			/// Run Clg

	rtcIntCt = 0;

	puts("<<< Real Time Clock demonstration start >>>");
	///Start OSC1
	CLG_WAIT = 0x2;						/// Wait 4096 cycle
	CLG_CTL |= 0x2;						/// Start OSC1
	for( i = 0 ; i < 61; i++ ){			/// Wait 4096 * 2000000/32768 Cycle
		for( n = 0 ; n < 4096; n++ ){
			asm ( "nop" );
		}
	}
	iRet = setRTC();
	if( iRet == 0 ){
		puts("*** Set RTC ERROR  ***");
		rnf_main = 0;
	} else {
		while(rtcIntCt != 10) {

		}
	}
	/// stop RTC
	StopRTC();
	iRet = InitRTCInt(0);
	if( iRet == 0 ){
		puts("***  Set Interrupt level ERROR  ***");
		rnf_main = 0;
	}
	iRet = DisableRTCInt(0x0008);
	if( iRet == 0 ){
		puts("***  Disable Int RTC ERROR  ***");
		rnf_main = 0;
	}
	puts("<<< Real Time Clock demonstration finish >>>");

	return ( rnf_main );
}
/****************************************************************************
 * setRTC function.
 *
 * @brief	First,The information acquired from Simulated I/O is set to each variable.
 * 			Next,RTC interruption is set up.
 *			then,RTC is started.
 *
 * @retval	RTC_SET_OK	Success.
 * @retval	RTC_SET_NG	Failed.
 ***************************************************************************/
static int setRTC(void)
{
	RTC_DATE_INFO 	rtcDate;						/// RTC count data
	char			inputString[5] = {'0','x','\0','\0','\0'};
	unsigned char 	tempTime;
	int				iRet = 1;
	unsigned char 	uc24mode;
	int				rnf;

	/// Set RTC date

	puts("> Input BCD format.");
	rtcDate.bcdmd = 1;
    /// set 24H/12H data
	puts("> 24H/12H (0/1) :");
	fgets(&inputString[2], sizeof(inputString) - 2, stdin);
	sscanf(inputString, "%x", (char*)&tempTime);
	uc24mode = tempTime;
	rtcDate.rtc24H = uc24mode;
	if( rtcDate.rtc24H == 1 ){
		puts("> AM/PM (0/1) :");
		fgets(&inputString[2], sizeof(inputString) - 2, stdin);
		sscanf(inputString, "%x", (char*)&tempTime);
		rtcDate.indicator = tempTime;
	} else{
		rtcDate.indicator = 0;		///Set Default Value
	}
    /// set hour data
	if( rtcDate.rtc24H == 1 ){
		puts("> Hour (1 - 12) :");
	}
	else{
		puts("> Hour (00 - 23) :");
	}
	fgets(&inputString[2], sizeof(inputString) - 2, stdin);
	sscanf(inputString, "%x", (char*)&tempTime);
	rtcDate.hour = ((tempTime / 16) << 4) | (tempTime % 16) ;

	/// set minute data
	puts("> Minute (00 - 59) :");
	fgets(&inputString[2], sizeof(inputString) - 2, stdin);
	sscanf(inputString, "%x", (char*)&tempTime);
	rtcDate.minute = ((tempTime / 16) << 4) | (tempTime % 16) ;

	/// set second data
	puts("> Second (00 - 59) :");
	fgets(&inputString[2], sizeof(inputString) - 2, stdin);
	sscanf(inputString, "%x", (char*)&tempTime);
	rtcDate.second = ((tempTime / 16) << 4) | (tempTime % 16) ;

	/// 24H/12H mode set
	rnf = InitRTC(&rtcDate);
	if (rnf == 0) {
		puts("***  Init RTC ERROR  ***");
		iRet = 0;
	}
	/// initialize RTC interrupt setting
	rnf = InitRTCInt(3);
	if (rnf == 0) {
		puts("***  Init RTC ERROR  ***");
		iRet = 0;
	}
	rnf = EnableRTCInt(0x0008);
	if (rnf == 0) {
		puts("***  Init RTC ERROR  ***");
		iRet = 0;
	}
	StartRTC();

	return( iRet );
}
/****************************************************************************
 * IntRtc function.
 *
 * @brief	First,It checks in 1HZ interruption.
 * 			Next,1HZ interruption flag is cleared.
 *			then,The number of times of interruption is displayed on Simulated I/O.
 *
 ***************************************************************************/
void IntRtc(void)
{
	char				cBuf[2];
	char				*cMesPnt;

	if( isRTCInt(0x0008) ){
		int					iRet;

		/// reset interrupt flag
		iRet = ClrRTCIntFlg(0x0008);
		if( iRet == 1 ){
			char				cDisp[50];
			int					iBufCnt;
			int					iCnt;

			/// count up the interrupt counter
			rtcIntCt++;
			cMesPnt = INTERRUPT_MESSAGE;
			CtoStr(cBuf,rtcIntCt);
			iBufCnt = 0;
			while( cMesPnt[iBufCnt] != '\0' ){
				cDisp[iBufCnt] = cMesPnt[iBufCnt];
				iBufCnt++;
			}
			iCnt = 0;
			while( cBuf[iCnt] != '\0' ){
				cDisp[iBufCnt] = cBuf[iCnt];
				iBufCnt++;
				iCnt++;
			}
			cDisp[iBufCnt] = '\0';
			puts(cDisp);
		}
	}
}
/****************************************************************************
 * CtoStr function.
 *
 * @brief	First,It is checked whether the argument cChgVal is larger than ten.
 *			then,Argument cChgVal is set as Argument cRes after changing into ASCII Code.
 *
 * @param	*cRes The storing pointer after ASCII Code conversion
 * @param	iChgVal Data changed into ASCII Code.
 ***************************************************************************/
static	void	CtoStr(char *cRes,int iChgVal)
{
    char	cAsctbl[10] = { '0','1','2','3','4','5','6','7','8','9' };

    if( iChgVal >= 0xa ){
    	cRes[0] = cAsctbl[1];
    	cRes[1] = cAsctbl[iChgVal - 0xa];
    	cRes[2] = '\0';
    } else {
    	cRes[0] = cAsctbl[iChgVal];
    	cRes[1] = '\0';
    }
}

