/****************************************************************************
 *
 * @file	main.c
 * @brief	This is sleep/halt-sample program for S1C17656 demonstration.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 ***************************************************************************/
/****************************************************************************
 * @mainpage	S1C17656 SLEEP/HALT
 *
 * @section		intro Introduction
 * Sample code for making it return from a HALT mode by T8 bit timer interrupt,
 * and making it return from a SLEEP mode by a port and RTC interruption.
 *
 * @subsection	step1	Step 1: Start OSC1.
 *						Starts the OSC1.
 * @subsection	step2	Step 2: The return from the HALT mode by T8 bit timer interruption.
 * 						Starts the T8 bit timer.
 *						It is made to change to HALT mode.
 *						Wait T8 bit timer Interrupt.
 *						Stops the T8 bit timer.
 * @subsection	step3	Step 3: It is made to return from SLEEP by port interruption.
 *						Port P00 input interruption is set up.
 *						It is made to change to SLEEP mode.
 *						Wait Port P00 Input Interrupt.
 *						Port P00 input interruption is forbidden.
 * @subsection	step4	Step 4: It is made to return from SLEEP by RTC interruption.
 *						Starts the RTC.
 *						It is made to change to SLEEP mode.
 *						Wait RTC Interrupt.
 *						Stops the RTC.
 * @section		install Installation
 * Start the IDE and import the "s1c17656_slphlt" project.
 *
 ***************************************************************************/

/// #include
#include <string.h>
#include <stdio.h>
#include <c17656_reg.h>
#include <init.h>
#include <port.h>
#include <t8.h>
#include <rtc.h>

#define	WAIT_8Cycle					(8)
#define	SLEEP_HALT_OK				(1)
#define	SLEEP_HALT_NG				(0)
#define	SLEEP_HALT_MAIN_LOOP_MAX	(5)
/// Prototype
int main(void);
static int chkHaltT8(void);
void IntT8(void)__attribute__ ((interrupt_handler));
static int chkSleepP00(void);
void IntP0x(void)__attribute__ ((interrupt_handler));
static int chkSleepRtc(void);
void IntRtc(void)__attribute__ ((interrupt_handler));

/// global
volatile int rtcIntCt;	///RTC Interrupt Counter
volatile int t8fFlg;
volatile int loopCount;
volatile int sw0Flag;
volatile int rtcFlag;
/****************************************************************************
 * Main function.
 *
 * @brief	First,A MISC register is set up.
 *			Second,OSC1B is started.
 *			Third,A chkHaltT8 function is called,
 *			HALT mode release by T8 bit timer is checked.
 * 			Next,A chkSleepP00 function is called,
 *			SLEEP mode release by Port P00 is checked.
 *			then,A chkSleepRtc function is called,
 *			SLEEP mode release by RTC is checked.
 * @retval	SLEEP_HALT_OK	Success.
 * @retval	SLEEP_HALT_NG	Failed.
 ***************************************************************************/
int main(void)
{
	int		i,n;
	int		iRet;
	int		iRet1;
	int		iRet2;
	int		iRet3;

	DebugModeMisc(0);				/// Misc Stop in Debug Mode
	DebugModePsc(0);				/// Prescaler Stop in Debug Mode
	SetClockGear(0);				/// Set Clock Gear *default=0x0
	ControlClg(3);					/// Run Clg

	printf("<<< Sleep/halt demonstration start >>>\n");
	///Start OSC1
	CLG_WAIT = 0x2;						/// Wait 4096 cycle
	CLG_CTL |= 0x2;						/// Start OSC1
	for( i = 0 ; i < 61; i++ ){			/// Wait 4096 * 2000000/32768 Cycle
		for( n = 0 ; n < 4096; n++ ){
			asm ( "nop" );
		}
	}
	iRet1 = chkHaltT8();						/// 8Bit Timer HALT Check
	iRet2 = chkSleepP00();						/// P00 SLEEP Check
	iRet3 = chkSleepRtc();						/// RTC SLEEP Check
	if ((iRet1 == 1) &&
			(iRet2 == 1) &&
				(iRet3 == 1)){
		iRet = 1;
	} else {
		iRet = 0;
	}
	puts("<<< Sleep/halt demonstration finish >>>");
	return (iRet);

}
/****************************************************************************
 * chkHaltT8 function.
 *
 * @brief	First,T8 bit timer is started.
 * 			Second,It is made to change to HALT mode.
 *			Next,Wait T8 bit timer Interrupt.
 *			then,T8 bit timer is stopped.
 * @retval	T8_SET_OK	Success.
 * @retval	T8_SET_NG	Failed.
 ***************************************************************************/
static int chkHaltT8(void)
{
	int		rnf;
	int		rnf_main = 1;

	/// T8F Initialize
	rnf = InitT8(0, 7, 0x80);
	if(rnf == 0){
		printf("***  InitT8 ERROR  ***\n");
		rnf_main = 0;
	}

	/// Interrupt Initialize
	rnf = InitT8Int(3);
	if(rnf == 0){
		printf("***  InitT8Int ERROR  ***\n");
		rnf_main = 0;
	}
	EnableT8Int();

	/// Timer Start
	StartT8();

	loopCount	= 0;
	while (loopCount != 5){
		printf("go to halt mode\n");
		asm ( "ei" );
		asm ( "halt" );
		asm ( "nop" );
		asm ( "di" );

		if (t8fFlg==1){
			printf("return from halt mode\n");
			loopCount++;
			t8fFlg = 0;
		}
	}

	/// Timer Stop
	StopT8();

	/// Interrupt Initialize
	rnf = InitT8Int( 0 );
	if( rnf == 0 ){
		printf("***  InitT8Int ERROR  ***\n");
		rnf_main = 0;
	}
	/// Interrupt disable
	DisableT8Int();

	return(rnf_main);
}
/****************************************************************************
 * IntT8 function.
 *
 * @brief	T8 bit timer interrupt routine.
 ***************************************************************************/
void IntT8(void)
{
	if (T8_INT0_T8IF){
		t8fFlg = 1;
	}
}
/****************************************************************************
 * chkSleepP00 function.
 *
 * @brief	First,Port P00 input interruption is set up.
 *			Second,It is made to change to SLEEP mode.
 * 			Next,Wait Port P00 Input Interrupt.
 *			then,Port P00 input interruption is forbidden.
 * @retval	PORT_SET_OK	Success.
 * @retval	PORT_SET_NG	Failed.
 ***************************************************************************/
static int chkSleepP00(void)
{
	int		rnf;
	int		rnf_main = 1;


	/// P0 interrupt all disable
	DisableP0Int(0xff);

	/// Chattering Filter Setup
	SetP0Chat(0, 0);

	/// Input Enable Setup
	SetPortInput(1, 0, 0);

	/// Port0 Interrupt Setup
	rnf = InitP0Int(3, 0x01);
	if( rnf == 0 ){
		printf("***  InitP0Int ERROR  ***\n");
		rnf_main = 0;
	}
	/// Port0 Interrupt enable
	EnableP0Int(0x01);

	sw0Flag = 0;
	printf("go to sleep mode(SW P00)\n");
	while (!sw0Flag){
		asm ( "ei" );
		asm ( "slp" );
		asm ( "nop" );
		asm ( "di" );
		if ( sw0Flag ){
			printf("return from sleep mode\n");
		}
	}
	/// Port0 interrupt disable
	DisableP0Int(0xFF);
	return(rnf_main);
}
/****************************************************************************
 * IntP0x function.
 *
 * @brief	P0x interrupt routine.
 ***************************************************************************/
void IntP0x(void)
{
	if (isP0Int(0x01)) {
		sw0Flag = 1;
	}
}
/****************************************************************************
 * chkSleepRtc function.
 *
 * @brief	First,RTC is started.
 * 			Second,It is made to change to SLEEP mode.
 *			Next,Wait RTC Interrupt.
 *			then,RTC is stopped.
 * @retval	RTC_SET_OK	Success.
 * @retval	RTC_SET_NG	Failed.
 ***************************************************************************/
static int chkSleepRtc(void)
{
	int				rnf;
	int				rnf_main = 1;
	RTC_DATE_INFO 	rtcDate;						/// RTC count data

	rtcDate.bcdmd = (unsigned char)1;
	rtcDate.rtc24H = (unsigned char)0;
	rtcDate.indicator = (unsigned char)0;
	rtcDate.hour = (unsigned char)0;
	rtcDate.minute = (unsigned char)0;
	rtcDate.second = (unsigned char)0;
	rnf = InitRTC(&rtcDate);
	if (rnf == 0) {
		puts("***  Init RTC ERROR  ***");
		rnf_main = 0;
	}
	rnf = InitRTCInt(3);
	if(rnf == 0){
		puts("***  Int RTC ERROR  ***");
		rnf_main = 0;
	}
	rnf = EnableRTCInt(0x0008);
	if(rnf == 0){
		puts("***  Int RTC ERROR  ***");
		rnf_main = 0;
	}
	StartRTC();
	rtcFlag = 0;
	printf("go to sleep mode(RTC)\n");
	while (!rtcFlag){
		asm ( "ei" );
		asm ( "slp" );
		asm ( "nop" );
		asm ( "di" );
		if ( rtcFlag ){
			printf("return from sleep mode\n");
		}
	}
	/// stop RTC
	StopRTC();
	rnf = InitRTCInt(0);
	if(rnf == 0){
		puts("***  Disable Int RTC ERROR  ***");
		rnf_main = 0;
	}
	rnf = DisableRTCInt(0x8);
	if(rnf == 0){
		puts("***  Disable Int RTC ERROR  ***");
		rnf_main = 0;
	}
	return( rnf_main );
}
/****************************************************************************
 * IntRtc function.
 *
 * @brief	First,It checks in 1HZ interruption.
 * 			then,1HZ interruption flag is cleared.
 *
 ***************************************************************************/
void IntRtc(void)
{

	if( isRTCInt(0x0008) ){
		int					iRet;

		/// reset interrupt flag
		iRet = ClrRTCIntFlg(0x0008);
		if( iRet == 1 ){
			rtcFlag = 1;
		}
	}
}
