/****************************************************************************
 *
 * @file	port.c
 * @brief	This is port driver program file for S1C17656.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 ***************************************************************************/

/* #include */
#include <c17656_reg.h>
#include <port.h>

/****************************************************************************
 * SetPortInput function.
 *
 * @brief	P Port is set up in the input mode and Pull Up enable.
 *
 * @param	inputPort0
 *	@li @c		P_BIT_X0	D0[PORT0]
 *	@li @c		P_BIT_X1	D1[PORT0]
 *	@li @c		P_BIT_X2	D2[PORT0]
 *	@li @c		P_BIT_X3	D3[PORT0]
 *	@li @c		P_BIT_X4	D4[PORT0]
 *	@li @c		P_BIT_X5	D5[PORT0]
 *	@li @c		P_BIT_X6	D6[PORT0]
 *	@li @c		P_BIT_X7	D7[PORT0]
 * @param	inputPort1
 *	@li @c		P_BIT_X0	D0[PORT0]
 *	@li @c		P_BIT_X2	D2[PORT1]
 *	@li @c		P_BIT_X3	D3[PORT2]
 *	@li @c		P_BIT_X4	D4[PORT1]
 *	@li @c		P_BIT_X5	D5[PORT1]
 *	@li @c		P_BIT_X6	D6[PORT1]
 *	@li @c		P_BIT_X7	D7[PORT1]
 * @param	inputPort2
 *	@li @c		P_BIT_X0	D0[PORT2]
 *	@li @c		P_BIT_X1	D1[PORT2]
 *	@li @c		P_BIT_X2	D2[PORT2]
 *	@li @c		P_BIT_X3	D3[PORT2]
 ***************************************************************************/
void SetPortInput( unsigned char inputPort0, unsigned char inputPort1, unsigned char inputPort2 )
{
	/// P0X Input and Pull Up Enable
	P0_IEN = inputPort0;
	P0_PU = inputPort0;

	/// P1X Input and Pull Up Enable
	P1_IEN = inputPort1;
	P1_PU = inputPort1;

	/// P2X Input and Pull Up Enable
	P2_IEN = inputPort2;
	P2_PU = inputPort2;
}

/****************************************************************************
 * SetPortOutput function.
 *
 * @brief	 P Port is set up in the output mode.
 *
 * @param	outputPort0
 *	@li @c		P_BIT_X0	D0[PORT0]
 *	@li @c		P_BIT_X1	D1[PORT0]
 *	@li @c		P_BIT_X2	D2[PORT0]
 *	@li @c		P_BIT_X3	D3[PORT0]
 *	@li @c		P_BIT_X4	D4[PORT0]
 *	@li @c		P_BIT_X5	D5[PORT0]
 *	@li @c		P_BIT_X6	D6[PORT0]
 *	@li @c		P_BIT_X7	D7[PORT0]
 * @param	outputPort1
 *	@li @c		P_BIT_X0	D0[PORT1]
 *	@li @c		P_BIT_X1	D1[PORT1]
 *	@li @c		P_BIT_X2	D2[PORT1]
 *	@li @c		P_BIT_X3	D3[PORT1]
 *	@li @c		P_BIT_X4	D4[PORT1]
 *	@li @c		P_BIT_X5	D5[PORT1]
 *	@li @c		P_BIT_X6	D6[PORT1]
 *	@li @c		P_BIT_X7	D7[PORT1]
 * @param	outputPort2
 *	@li @c		P_BIT_X0	D0[PORT2]
 *	@li @c		P_BIT_X1	D1[PORT2]
 *	@li @c		P_BIT_X2	D2[PORT2]
 *	@li @c		P_BIT_X3	D3[PORT2]
 ***************************************************************************/
void SetPortOutput(	unsigned char outputPort0, unsigned char outputPort1, unsigned char outputPort2 )
{
	/// P0X Output Enable and Set a Output Data High Level
	P0_OUT = outputPort0;
	P0_OEN = outputPort0;

	/// P1X Output Enable and Set a Output Data High Level
	P1_OUT = outputPort1;
	P1_OEN = outputPort1;

	/// P2X Output Enable and Set a Output Data High Level
	P2_OUT = outputPort2;
	P2_OEN = outputPort2;

}

/****************************************************************************
 * SetPortOutputData function.
 *
 * @brief	the data(Low / High) output of P Port.
 *
 * @param	portCh
 *	@li @c		PORT_PORTNO_0	PORT 0
 *	@li @c		PORT_PORTNO_1	PORT 1
 *	@li @c		PORT_PORTNO_2	PORT 2
 * @param	outputData
 *	@li @c		PORT_MAIN_PORT_OUT_LOW		Low
 *	@li @c		PORT_MAIN_PORT_OUT_HIGH		High
 *
 * @retval	PORT_SET_NG		Failed.
 * @retval	PORT_SET_OK		Success.
 ***************************************************************************/
int SetPortOutputData(	unsigned char portCh, unsigned char outputData )
{
	/// Check Parameter
	if( portCh > 2){
		return 0;
	}

	/// Set Port Output Data
	if( portCh == 0 ) {
		P0_OUT = outputData;
	} else if( portCh == 1 ) {
		P1_OUT = outputData;
	} else{
		P2_OUT = outputData;
	}

	return 1;
}

/****************************************************************************
 * SetP0Chat function.
 *
 * @brief	Port00-03 and Port04-07 is set up in the chattering time.
 *
 * @param	p0p3Chat
 *	@li @c		P_CAHT_TIME_NONE		NONE
 *	@li @c		P_CAHT_TIME_PCLK_256	256/PCLK
 *	@li @c		P_CAHT_TIME_PCLK_512	512/PCLK
 *	@li @c		P_CAHT_TIME_PCLK_1024	1024/PCLK
 *	@li @c		P_CAHT_TIME_PCLK_2048	2048/PCLK
 *	@li @c		P_CAHT_TIME_PCLK_4096	4096/PCLK
 *	@li @c		P_CAHT_TIME_PCLK_8192	8192/PCLK
 *	@li @c		P_CAHT_TIME_PCLK_16384	16384/PCLK
 * @param	p4p7Chat
 *	@li @c		P_CAHT_TIME_NONE		NONE
 *	@li @c		P_CAHT_TIME_PCLK_256	256/PCLK
 *	@li @c		P_CAHT_TIME_PCLK_512	512/PCLK
 *	@li @c		P_CAHT_TIME_PCLK_1024	1024/PCLK
 *	@li @c		P_CAHT_TIME_PCLK_2048	2048/PCLK
 *	@li @c		P_CAHT_TIME_PCLK_4096	4096/PCLK
 *	@li @c		P_CAHT_TIME_PCLK_8192	8192/PCLK
 *	@li @c		P_CAHT_TIME_PCLK_16384	16384/PCLK
 ***************************************************************************/
void SetP0Chat(unsigned char p0p3Chat, unsigned char p4p7Chat)
{
	/// Chattering Filter Time P00-03
	/// Chattering Filter Time P04-07
	P0_CHAT = (p0p3Chat | (p4p7Chat<<4));
}

/****************************************************************************
 * InitP0Int function.
 *
 * @brief	 Port0 set up in the Port0 Interrupt.
 *
 * @param	intLevel
 *	@li @c		PORT_INTLV_0		Interrupt Level 0
 *	@li @c		PORT_INTLV_1		Interrupt Level 1
 *	@li @c		PORT_INTLV_2		Interrupt Level 2
 *	@li @c		PORT_INTLV_3		Interrupt Level 3
 *	@li @c		PORT_INTLV_4		Interrupt Level 4
 *	@li @c		PORT_INTLV_5		Interrupt Level 5
 *	@li @c		PORT_INTLV_6		Interrupt Level 6
 *	@li @c		PORT_INTLV_7		Interrupt Level 7
 * @param	p0Edge
 *	@li @c		P_BIT_X0	D0[PORT0]
 *	@li @c		P_BIT_X1	D1[PORT0]
 *	@li @c		P_BIT_X2	D2[PORT0]
 *	@li @c		P_BIT_X3	D3[PORT0]
 *	@li @c		P_BIT_X4	D4[PORT0]
 *	@li @c		P_BIT_X5	D5[PORT0]
 *	@li @c		P_BIT_X6	D6[PORT0]
 *	@li @c		P_BIT_X7	D7[PORT0]
 *
 * @retval	PORT_SET_NG		Failed.
 * @retval	PORT_SET_OK		Success.
 ***************************************************************************/
int InitP0Int( unsigned char intLevel, unsigned char p0Edge )
{
	/// Check parameter
	if( intLevel > 7 ){
		return 0;
	}

	/// Set an Interrupt Level
	ITC_LV0 = intLevel;

	/// Set an Interrupt Edge
	P0_EDGE = p0Edge;

	return 1;
}

/****************************************************************************
 * EnableP0Int function.
 *
 * @brief	 Port enable in the Port0 Interrupt.
 *
 * @param	p0Imsk
 *	@li @c		P_BIT_X0	D0[PORT0]
 *	@li @c		P_BIT_X1	D1[PORT0]
 *	@li @c		P_BIT_X2	D2[PORT0]
 *	@li @c		P_BIT_X3	D3[PORT0]
 *	@li @c		P_BIT_X4	D4[PORT0]
 *	@li @c		P_BIT_X5	D5[PORT0]
 *	@li @c		P_BIT_X6	D6[PORT0]
 *	@li @c		P_BIT_X7	D7[PORT0]
 ***************************************************************************/
void EnableP0Int( unsigned char p0Imsk)
{
	/// Reset Interrupt Flag
	ClrP0IntFlg(p0Imsk);

	/// Set an Interrupt Enable
	P0_IMSK = p0Imsk;
}

/****************************************************************************
 * DisableP0Int function.
 *
 * @brief	 Port disable in the Port0 Interrupt.
 *
 * @param	p0Imsk
 *	@li @c		P_BIT_X0	D0[PORT0]
 *	@li @c		P_BIT_X1	D1[PORT0]
 *	@li @c		P_BIT_X2	D2[PORT0]
 *	@li @c		P_BIT_X3	D3[PORT0]
 *	@li @c		P_BIT_X4	D4[PORT0]
 *	@li @c		P_BIT_X5	D5[PORT0]
 *	@li @c		P_BIT_X6	D6[PORT0]
 *	@li @c		P_BIT_X7	D7[PORT0]
 ***************************************************************************/
void DisableP0Int( unsigned char p0Imsk )
{
	/// Set an Interrupt Enable
	P0_IMSK = ~(p0Imsk);

	/// Reset Interrupt Flag
	ClrP0IntFlg(p0Imsk);
}


/****************************************************************************
 * isP0Int function.
 *
 * @brief	 Check P0 interrupt and return main function.
 *
 * @param	Intbit
 *	@li @c		P_BIT_X0	D0[PORT0]
 *	@li @c		P_BIT_X1	D1[PORT0]
 *	@li @c		P_BIT_X2	D2[PORT0]
 *	@li @c		P_BIT_X3	D3[PORT0]
 *	@li @c		P_BIT_X4	D4[PORT0]
 *	@li @c		P_BIT_X5	D5[PORT0]
 *	@li @c		P_BIT_X6	D6[PORT0]
 *	@li @c		P_BIT_X7	D7[PORT0]
 *
 * @retval	PORT_NOTOCCURRED	Not occurred
 * @retval	PORT_OCCURRED		Occurred
 ***************************************************************************/
int isP0Int(unsigned char Intbit)
{
	unsigned char intFlag;

	/// Port interrupt Flag
	intFlag = P0_IFLG;

	/// Check Interrupt Flag
	if ( (intFlag & Intbit) == Intbit ){
		/// Reset P0x Interrupt Flag
		ClrP0IntFlg(Intbit);
		return 1;
	} else {
		return 0;
	}
}

/****************************************************************************
 * ClrP0IntFlg function.
 *
 * @brief	 Clear P00 interrupt flag.
 *
 * @param	Intbit
 *	@li @c		P_BIT_X0	D0[PORT0]
 *	@li @c		P_BIT_X1	D1[PORT0]
 *	@li @c		P_BIT_X2	D2[PORT0]
 *	@li @c		P_BIT_X3	D3[PORT0]
 *	@li @c		P_BIT_X4	D4[PORT0]
 *	@li @c		P_BIT_X5	D5[PORT0]
 *	@li @c		P_BIT_X6	D6[PORT0]
 *	@li @c		P_BIT_X7	D7[PORT0]
 ***************************************************************************/
void ClrP0IntFlg( unsigned char Intbit )
{
	/// Reset Interrupt Flag
	P0_IFLG = Intbit;
}

