/*******************************************************************************
 *
 * @file	rtc.c
 * @brief	This is rtc program for S1C17656 demonstration.
 *
 * @par		Copyright:
 * 			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 ******************************************************************************/

#include <rtc.h>
#include <c17656_reg.h>
#include <rtc_reg.h>

int	InitRTC(RTC_DATE_INFO *pDate);
void StartRTC(void);
void StopRTC(void);
int	InitRTCInt( unsigned char intLevel );
int	EnableRTCInt( unsigned short intSelect );
int	DisableRTCInt( unsigned short intSelect );
int	isRTCInt( unsigned short intType );
int	ClrRTCIntFlg( unsigned short intType );

/****************************************************************************
 * InitRTC function.
 *
 * @brief	First,The value of Argument pDate is checked.
 *			Next,RTC is stopped.
 *			then, The argument pDate is set as a register.
 *
 * @param	*pDate RTC setting data
 *
 * @retval	RTC_SET_OK	Success.
 * @retval	RTC_SET_NG	Failed.
 *
 ***************************************************************************/
int	InitRTC(RTC_DATE_INFO *pDate)
{
	int		iRet;

	/// Check Parameter
	if(pDate->bcdmd == 1){
		if((((pDate->hour) & 0x0F) > 9) || ((pDate->hour) > 0x23)){

			iRet =  0;
			return iRet;
		}
		if((((pDate->minute) & 0x0F) > 9) || ((pDate->minute) > 0x59)){

			iRet =  0;
			return iRet;
		}

		if((((pDate->second) & 0x0F) > 9) || ((pDate->second) > 0x59)){

			iRet =  0;
			return iRet;
		}

	}
	else if(pDate->bcdmd == 0){
		if((pDate->hour) > 0x17){

			iRet =  0;
			return iRet;
		}
		if((pDate->minute) > 0x3B){

			iRet =  0;
			return iRet;
		}
		if((pDate->second) > 0x3B){

			iRet =  0;
			return iRet;
		}
	}

	if((pDate->bcdmd > 1) ||
			(pDate->rtc24H > 1)	||
			((pDate->indicator) > 1) ) {
		iRet =  0;
	}
	else {
		StopRTC();
		/// Set BCD Mode
		/// Set 24H/12H
		RTC_CTL = ((pDate->bcdmd << 5) | (pDate->rtc24H << 4));

		/// Set Second
		/// Set Minute
		RTC_MS = ((pDate->second) | (pDate->minute << 8));

		/// Set Hour
		RTC_H = pDate->hour;

		if(pDate->bcdmd == 1){
			if(pDate->rtc24H == 1){
				if((pDate->hour > 0x12) || (pDate->hour == 0)){
					iRet =  0;
					return iRet;
				}
				else{
					RTC_H = ((RTC_H & 0xff7f) | (pDate->indicator << 7));
				}
			}
		}
		else if(pDate->bcdmd == 0){
			if(pDate->rtc24H == 1){
				if((pDate->hour > 0xc) || (pDate->hour == 0)){
					iRet =  0;
					return iRet;
				}
				else{
					RTC_H = ((RTC_H & 0xff7f) | (pDate->indicator << 7));
				}
			}

		}
#if 0
		RTC_CTL_BCDMD = pDate->bcdmd;		/// Set BCD Mode
		RTC_CTL_RTC24H = pDate->rtc24H;		/// Set 24H/12H
		RTC_MS_RTCSEC = pDate->second;		/// Set Second
		RTC_MS_RTCMIN = pDate->minute;		/// Set Minute
		RTC_H_RTCHOUR = pDate->hour;		/// Set Hour
		if( pDate->rtc24H == 1 ){
			RTC_H_AMPM = pDate->indicator;	/// Set AM/PM
		}
#endif

		iRet =  1;
	}
	return iRet;
}
/****************************************************************************
 * StartRTC function.
 *
 * @brief	RTC is started.
 ***************************************************************************/
void	StartRTC( void )
{
	RTC_CTL |= 0x0001;
}
/****************************************************************************
 * StopRTC function.
 *
 * @brief	RTC is stopped.
 ***************************************************************************/
void	StopRTC( void )
{
	RTC_CTL &= 0xfffe;
}
/****************************************************************************
 * InitRTCInt function.
 *
 * @brief	First,The value of Argument intLevel is checked.
 *			then, The argument intLevel is set as a register.
 *
 * @param	intLevel Interrupt-priorities data
 *
 * @retval	RTC_SET_OK	Success.
 * @retval	RTC_SET_NG	Failed.
 *
 ***************************************************************************/
int	InitRTCInt( unsigned char intLevel )
{
	int		iRet;
	/// Check Parameter
	if( intLevel > 7 ){
		iRet =  0;
	} else {
		ITC_LV2 = intLevel;
		iRet =  1;
	}
	return iRet;
}
/****************************************************************************
 * EnableRTCInt function.
 *
 * @brief	First,The value of Argument intSelect is checked.
 *			then, The argument intSelect is set as a register.
 *
 * @param	intSelect Interruption enable data
 *
 * @retval	RTC_SET_OK	Success.
 * @retval	RTC_SET_NG	Failed.
 *
 ***************************************************************************/
int	EnableRTCInt( unsigned short intSelect )
{
	int		iRet;
	/// Check Parameter
	if((intSelect & ~0x03ff) != 0) {
		iRet = 0;
	} else {
		RTC_IEN |= intSelect;
		iRet = 1;
	}
	return iRet;
}
/****************************************************************************
 * DisableRTCInt function.
 *
 * @brief	First,The value of Argument intSelect is checked.
 *			then, The argument intSelect is set as a register.
 *
 * @param	intSelect Interruption disable data
 *
 * @retval	RTC_SET_OK	Success.
 * @retval	RTC_SET_NG	Failed.
 *
 ***************************************************************************/
int	DisableRTCInt( unsigned short intSelect )
{
	int		iRet;
	/// Check Parameter
	if((intSelect & ~0x03ff) != 0) {
		iRet = 0;
	} else {
		RTC_IEN &= (~intSelect & 0x03ff);
		iRet = 1;
	}
	return iRet;
}
/****************************************************************************
 * isRTCInt function.
 *
 * @brief	First,The value of Argument intType is checked.
 *			then, A register is read and a value is checked.
 *
 * @param	intType Interruption check data
 *
 * @retval	RTC_SET_OK	Success.
 * @retval	RTC_SET_NG	Failed.
 *
 ***************************************************************************/
int	isRTCInt( unsigned short intType )
{
	int		iRet;
	/// Check Parameter
	if((intType & ~0x03ff) != 0) {
		iRet = 0;
	} else {
		if( RTC_IFLG & intType ){
			iRet = 1;
		} else {
			iRet = 0;
		}
	}
	return iRet;
}
/****************************************************************************
 * ClrRTCIntFlg function.
 *
 * @brief	First,The value of Argument intType is checked.
 *			then, The argument intType is set as a register.
 *
 * @param	intType Interruption flag clear data
 *
 * @retval	RTC_SET_OK	Success.
 * @retval	RTC_SET_NG	Failed.
 *
 ***************************************************************************/
int	ClrRTCIntFlg( unsigned short intType )
{
	int		iRet;
	/// Check Parameter
	if((intType & ~0x03ff) != 0) {
		iRet = 0;
	} else {
		RTC_IFLG = intType;
		iRet = 1;
	}
	return iRet;
}

