/****************************************************************************
 *
 * @file	t8.c
 * @brief	This is t8 driver program file for S1C17656.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 ***************************************************************************/

/* #include */
#include <c17656_reg.h>
#include <t8.h>

/****************************************************************************
 * InitT8 function.
 *
 * @brief	T8 timer initialize.
 *
 * @param	cntMode
 *	@li @c		T8_REPEAT	REPEAT mode
 *	@li @c		T8_ONESHOT	ONESHOT mode
 * @param	cntClk
 *	@li @c		T8_CLOCK_DIV_1		1/1
 *	@li @c		T8_CLOCK_DIV_2		1/2
 *	@li @c		T8_CLOCK_DIV_4		1/4
 *	@li @c		T8_CLOCK_DIV_8		1/8
 *	@li @c		T8_CLOCK_DIV_16		1/16
 *	@li @c		T8_CLOCK_DIV_32		1/32
 *	@li @c		T8_CLOCK_DIV_64		1/64
 *	@li @c		T8_CLOCK_DIV_128	1/128
 *	@li @c		T8_CLOCK_DIV_256	1/256
 *	@li @c		T8_CLOCK_DIV_512	1/512
 *	@li @c		T8_CLOCK_DIV_1024	1/1024
 *	@li @c		T8_CLOCK_DIV_2048	1/2048
 *	@li @c		T8_CLOCK_DIV_4096	1/4096
 *	@li @c		T8_CLOCK_DIV_8192	1/8192
 *	@li @c		T8_CLOCK_DIV_16384	1/16384
 * @param	Count
 *	@li @c		0x00-0xff	Count data
 *
 * @retval	T8_SET_NG	Failed.
 * @retval	T8_SET_OK	Success.
 ***************************************************************************/
int InitT8( unsigned short cntMode, unsigned short cntClk, unsigned short Count )
{
	/// Parameter Check
	if( cntMode > 1 ||
		cntClk  > 0xE ||
		Count > 0xff	){
			return 0;
	}

	/// Timer Stop
	StopT8();

	/// Count Mode Select
	T8_CTL0 = (cntMode<<4);

	/// Count Clock Select
	T8_CLK0 = cntClk;

	/// Count Reload Data Set
	T8_TR0 = Count;

	/// Timer Reset
	T8_CTL0 |= 0x0002;

	return 1;
}

/****************************************************************************
 * GetT8Count function.
 *
 * @brief	Get T8 count data.
 *
 * @return T8_TC0_TC	Return T8 count data.
 ***************************************************************************/
unsigned short GetT8Count(void)
{
	return ( (unsigned short)T8_TC0 );
}

/****************************************************************************
 * StartT8 function.
 *
 * @brief	T8 timer start.
 ***************************************************************************/
void StartT8(void)
{
	/// Timer start
	T8_CTL0 |= 0x0001;
}

/****************************************************************************
 * StopT8 function.
 *
 * @brief	T8 timer stop.
 ***************************************************************************/
void StopT8(void)
{
	/// Timer stop
	T8_CTL0 &= 0xFFFE;
}

/****************************************************************************
 * InitT8Int function.
 *
 * @brief	 Initialize T8 timer interrupt.
 *
 * @param	intLevel
 *	@li @c		T8_INTLV_0		Interrupt Level 0
 *	@li @c		T8_INTLV_1		Interrupt Level 1
 *	@li @c		T8_INTLV_2		Interrupt Level 2
 *	@li @c		T8_INTLV_3		Interrupt Level 3
 *	@li @c		T8_INTLV_4		Interrupt Level 4
 *	@li @c		T8_INTLV_5		Interrupt Level 5
 *	@li @c		T8_INTLV_6		Interrupt Level 6
 *	@li @c		T8_INTLV_7		Interrupt Level 7
 *
 * @retval	T8_SET_NG	Failed.
 * @retval	T8_SET_OK	Success.
 ***************************************************************************/
int InitT8Int( unsigned char intLevel )
{
	/// Parameter Check
	if( intLevel > 7 ){
			return 0;
	}

	/// Interrupt Level Set
	ITC_LV5 = intLevel;

	return 1;
}

/****************************************************************************
 * EnableT8Int function.
 *
 * @brief	 T8 timer interrupt enable.
 ***************************************************************************/
void EnableT8Int( void )
{
	/// Interrupt Reset
	ClrT8IntFlg();

	/// Interrupt enable Set
	T8_INT0 |= 0x0100;

}

/****************************************************************************
 * DisableT8Int function.
 *
 * @brief	 T8 timer interrupt disable.
 ***************************************************************************/
void DisableT8Int( void )
{
	/// Interrupt enable Set
	T8_INT0 &= 0xFEFF;

	/// Interrupt Reset
	ClrT8IntFlg();
}

/****************************************************************************
 * isT8Int function.
 *
 * @brief	 Check T8 interrupt occurred.
 *
 * @retval	PORT_NOTOCCURRED	Not occurred
 * @retval	PORT_OCCURRED		Occurred
 ***************************************************************************/
int isT8Int( void )
{
	/// Interrupt Check
	if( T8_INT0_T8IF != T8_OCCURRED ){
		return 0;
	}

	return 1;
}

/****************************************************************************
 * ClrT8IntFlg function.
 *
 * @brief	 Clear T8 timer interrupt flag.
 ***************************************************************************/
void ClrT8IntFlg( void )
{
	/// Interrupt Reset
	T8_INT0_T8IF = 1;
}

