/****************************************************************************
 *
 * @file	snd.c
 * @brief	This is snd driver program file for S1C17656.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 ***************************************************************************/

/* #include */
#include <c17656_reg.h>
#include <snd.h>

/****************************************************************************
 * InitSND function.
 *
 * @brief	SND initialize.
 *
 * @param	 *mode
 * @li	mode->bzmd
 *	 	@c	SND_NORMALMD		Normal mode
 *	 	@c	SND_ONTSHOTMD		One-shot mode
 *	 	@c	SND_ENVELOPEMD		Envelope mode
 * @li	mode->bztm
 *	 	@c	SND_15630_TIME		15.63 msec
 *	 	@c	SND_31250_TIME		31.25 msec
 *	 	@c	SND_62500_TIME		62.50 msec
 *	 	@c	SND_125000_TIME		125.0 msec
 * @li  mode->bzfq
 *	   	@c 	SND_4096000_FRQ		4096.0 Hz
 *	   	@c 	SND_3276800_FRQ		3276.8 Hz
 *	   	@c 	SND_2730700_FRQ		2730.7 Hz
 *	   	@c 	SND_2340600_FRQ		2340.6 Hz
 *	   	@c 	SND_2048000_FRQ		2048.0 Hz
 *	   	@c 	SND_1638400_FRQ		1638.4 Hz
 *	   	@c 	SND_1365300_FRQ		1365.3 Hz
 *	   	@c 	SND_1170300_FRQ		1170.3 Hz
 * @li	mode->duty
 *		@c	SND_BZDUTY_1		Buzzer duty ratio 1
 *		@c	SND_BZDUTY_2		Buzzer duty ratio 2
 *		@c	SND_BZDUTY_3		Buzzer duty ratio 3
 *		@c	SND_BZDUTY_4		Buzzer duty ratio 4
 *		@c	SND_BZDUTY_5		Buzzer duty ratio 5
 *		@c	SND_BZDUTY_6		Buzzer duty ratio 6
 *		@c	SND_BZDUTY_7		Buzzer duty ratio 7
 *		@c	SND_BZDUTY_8		Buzzer duty ratio 8
 *
 * @retval	SND_NG	Failed.
 * @retval	SND_OK	Success.
 ***************************************************************************/
int InitSND( SND_BZ_MODE *mode )
{
	/// Check parameter
	if( mode->bzmd > 2	||
		mode->bztm  > 3	||
		mode->bzfq > 7	||
		mode->duty > 7){
			return 0;
	}

	/// Stop SND
	StopSND();

	/// Set SND buzzer mode and buzzer time
	SND_CTL = ((mode->bzmd << 2) | (mode->bztm << 4));

//	SND_CTL_BZMD = mode->bzmd;
//	SND_CTL_BZTM = mode->bztm;

	/// Set SND buzzer frequency
	SND_BZFQ = mode->bzfq;

	/// Set SND buzzer duty ratio
	SND_BZDT = mode->duty;

	return 1;
}

/****************************************************************************
 * StartSND function.
 *
 * @brief	SND start.
 ***************************************************************************/
void StartSND(void)
{
	/// Start SND buzzer output
	SND_CTL |= 0x01;
}

/****************************************************************************
 * StopSND function.
 *
 * @brief	SND stop.
 ***************************************************************************/
void StopSND(void)
{
	/// Stop SND buzzer output
	SND_CTL &= 0xfe;
}

