/********************************************************************************
 *
 * @file	main.c
 * @brief	This is spi-sample program for S1C17656 demonstration.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 *******************************************************************************/
/********************************************************************************
 * @mainpage	S1C17656 SPI
 *
 * @section		intro Introduction
 * Sample code transmit data of 8bytes to SPI SLAVE.
 * The code receive data of 8bytes from SPI SLAVE.
 * When wait SPI interrupt, the code changes Halt mode of CPU,
 * reduce consumption electricity.
 *
 * @subsection	step1	Step 1: Start SPI.
 *						Start SPI, and Initialize SPI MASTER.
 *						Transmit ASCII data "FROM MST" of 8bytes to SPI SLAVE.
 * @subsection	step2	Step 2: Receive data.
 *						Output SPI clock to SPI SLAVE and Receive data,
 *						if finish to transmit data to SPI SLAVE.
 * @subsection	step4	Step 3: Display data.
 *						Receive data from SPI SLAVE, display to simulated I/O window.
 * @subsection	step4	Step 4: End sample code.
 *						End sample code.
 *
 * @section		install Installation
 * Start the IDE and import the "s1c17656_spi" project.
 *
 *******************************************************************************/

/// #include
#include <string.h>
#include <stdio.h>
#include <c17656_reg.h>
#include <init.h>
#include <spi.h>
#include <t8.h>

/// Prototype
int main(void);
void IntSPI(void)__attribute__ ((interrupt_handler));

#define	DATALEN	( 9 )							///< Max status
/// global
#ifndef SLAVE
	unsigned char sendData[DATALEN] = "FROM MST";
#else
	unsigned char sendData[DATALEN] = "FROM SLV";
#endif // SLAVE
unsigned char receiveData[DATALEN];
volatile unsigned short sendDataIndex	= 0;
volatile unsigned short reciveDataIndex	= 0;

/****************************************************************************
 * Main function.
 *
 * @retval	1	Normality
 * @retval	0	Abnormality
 ***************************************************************************/
int main(void)
{
	/// Initial Parameter
	int rnf;
	int rnf_main = 1;
	SPI_INIT_PARAM param;
	/// Initialize SPI status
	param.selectClk = 1;		/// SPI clock source
	param.selectMLSB = 0;		/// LSB/MSB first mode
	param.selectPhase = 1;		/// clock phase
	param.selectPolarity = 1;	/// clock polarity
#ifndef SLAVE
	param.selectMode = 1;		/// Master/Slave mode
#else
	param.selectMode = 0;		/// Master/Slave mode
#endif // SLAVE

	DebugModeMisc( 0 );			/// Misc Stop in Debug Mode
	DebugModePsc( 0 );			/// Prescaler Stop in Debug Mode
	SetClockGear( 0 );			/// Set Clock Gear *default=0x0
	ControlClg( 3 );			/// Run Clg

#ifndef SLAVE
	printf( "<<< SPI master demonstration start >>>\n" );
#else
	printf( "<<< SPI slave demonstration start >>>\n" );
#endif

	/// Display transmit data
	printf( "Transmit data : %s\n", sendData );

#ifndef SLAVE
	/// T8F Initialize
	rnf = InitT8( 0, 2, 0xff );
	if( rnf == 0 ){
		printf("***  InitT8 ERROR  ***\n");
		rnf_main = 0;
	}
	/// Timer Start
	StartT8();

	P00_03PMUX &= 0xfc;
	P0_OEN = 1;
	P0_OUT = 0;
#endif // SLAVE

	asm ( "di" );
	/// initializes SPI
	/// Set SPI parameter
	rnf = InitSPI( &param );
	if( rnf == 0 ){
		printf("***  InitSPI ERROR  ***\n");
		rnf_main = 0;
	}
	/// Set SPI interrupt level
	rnf = InitSPIInt( 3 );
	if( rnf == 0 ){
		printf("***  InitSPIInt ERROR  ***\n");
		rnf_main = 0;
	}
	/// SPI interrupt enable( Transmit and Receive interrupt )
	rnf = EnableSPIInt( 0x30 );
	if( rnf == 0 ){
		printf("***  EnableSPIInt ERROR  ***\n");
		rnf_main = 0;
	}
	/// Start SPI
	StartSPI();

	/// Set first sending
	SendDataSPI( sendData[sendDataIndex++] );



	/// wait till all data is sent, all data is received.
	while ( ((9 - 1) > sendDataIndex) || ((9 - 1) > reciveDataIndex) )
	{
		asm ( "ei" );
		asm ( "halt" );
		asm ( "nop" );
		asm ( "di" );
	}


	/// wait for the start of the last data transmission
	while ( isSPIInt( 0x01 ) != 1 ){
		;
	}
	/// wait for the end of the last data transmission
	while ( isSPIInt( 0x04 ) != 0 ){
		;
	}

#ifndef SLAVE
	P0_OUT = 1;
#endif

	/// Stop SPI
	StopSPI();
	/// SPI interrupt disable
	rnf = DisableSPIInt( 0x30 );
	if( rnf == 0 ){
		printf("***  DisableSPIInt ERROR  ***\n");
		rnf_main = 0;
	}
	/// Set SPI interrupt level 0
	rnf = InitSPIInt( 0 );
	if( rnf == 0 ){
		printf("***  InitSPIInt ERROR  ***\n");
		rnf_main = 0;
	}

	/// Stop T8 timer
	StopT8();

	/// Display received data
	printf( "Received data : %s\n", receiveData );

#ifndef SLAVE
	printf( "<<< SPI master demonstration finish >>>\n" );
#else
	printf( "<<< SPI slave demonstration finish >>>\n" );
#endif

	return ( rnf_main );
}

/****************************************************************************
 * IntSPI function.
 *
 * @brief	 SPI interrupt routine.
 ***************************************************************************/
void IntSPI(void)
{
	if ( isSPIInt( 0x01 ) == 1 ) {
		/// Send Buffer Empty interrupt
		if ( (9 - 1) > sendDataIndex ) {
			/// Send data
			SendDataSPI( sendData[sendDataIndex] );
			sendDataIndex++;
		}
	}

	if ( isSPIInt( 0x02 ) == 1 ) {
		/// Receive Buffer Full interrupt
		if ( (9 - 1) > reciveDataIndex ) {
			/// Get receive data
			receiveData[reciveDataIndex] = ReceiveDataSPI();
			reciveDataIndex++;
		}
	}
}

