/****************************************************************************
 *
 * @file	spi.c
 * @brief	This is spi driver program file for S1C17656.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 ***************************************************************************/

#include <c17656_reg.h>
#include <spi.h>

/****************************************************************************
 * InitSPI function.
 *
 * @brief	SPI initialize.
 *
 * @param	 *param
 *  @li  param->selectClk
 *	   	@c 	SPICTLMCLK_PCLK			PCLK/4
 *   	@c 	SPICTLMCLK_T8CH0		T8 timer
 * @li	param->selectMLSB
 *	 	@c	SPICTLMLSB_MSB			MSB
 *	 	@c	SPICTLMLSB_LSB			LSB
 * @li	param->selectPhase
 *	 	@c	SPICTLCPHA_DATAIN		Data In
 *	 	@c	SPICTLCPHA_DATAOUT		Data Out
 * @li	param->selectPolarity
 *	 	@c	SPICTLCPOL_ACTIVEH		Active high
 *	 	@c	SPICTLCPOL_ACTIVEL		Active low
 * @li	param->selectMode
 *	 	@c	SPICTLMSSL_SLAVE		Slave mode
 *	 	@c	SPICTLMSSL_MASTER		Master mode
 *
 * @retval	SPI_NG	Failed.
 * @retval	SPI_OK	Success.
 ***************************************************************************/
int InitSPI( SPI_INIT_PARAM *param )
{
	/// Check Parameter
	if( (param->selectClk		> 1) ||
		(param->selectMLSB		> 1) ||
		(param->selectPhase		> 1) ||
		(param->selectPolarity	> 1) ||
		(param->selectMode		> 1)){
			return 0;
	}

	/// Stop SPI
	StopSPI();

	/// Set SPI mode of PORT MUX function
	if( param->selectMode == 0 ){
		P04_07PMUX |= 0x08;
	}
	P04_07PMUX |= 0xa0;
	P10_13PMUX |= 0x02;

	SPI_CTL0 = ((param->selectClk << 9)		|
				(param->selectMLSB << 8)	|
				(param->selectPhase << 3)	|
				(param->selectPolarity << 2)|
				(param->selectMode << 1));

#if 0
	/// Set clock source
	SPI_CTL0_MCLK = param->selectClk;

	/// Set LSB/MSB first mode
	SPI_CTL0_MLSB = param->selectMLSB;

	/// Set clock phase
	SPI_CTL0_CPHA = param->selectPhase;

	/// Set clock polarity
	SPI_CTL0_CPOL = param->selectPolarity;

	/// Set Master/Slave mode
	SPI_CTL0_MSSL = param->selectMode;
#endif

	return 1;
}

/****************************************************************************
 * SendDataSPI function.
 *
 * @brief	SPI transmit data write.
 *
 * @param	data
 *	@li @c		0x00-0xff	transmit data
 ***************************************************************************/
void SendDataSPI(unsigned char data)
{
	/// SPI Transmit Data Register
	SPI_TXD0 = data;
}

/****************************************************************************
 * ReceiveDataSPI function.
 *
 * @brief	SPI receive data read.
 *
 * @return	data	UART receive data.
 ***************************************************************************/
unsigned char ReceiveDataSPI(void)
{
	/// UART receive data read
	return ( SPI_RXD0 );
}

/****************************************************************************
 * StartSPI function.
 *
 * @brief	Set SPI enable.
 ***************************************************************************/
void StartSPI(void)
{
	/// Set SPI enable
	SPI_CTL0 |= 0x0001;
}

/****************************************************************************
 * StopSPI function.
 *
 * @brief	Set SPI disable.
 ***************************************************************************/
void StopSPI(void)
{
	/// Set SPI disable
	SPI_CTL0 &= 0xfffe;
}


/****************************************************************************
 * EnableSPIInt function.
 *
 * @brief	SPI interrupt enable.
 *
 * @param	intSelect
 *	@li @c		SPICTLSPTIE			Transmit data buffer empty interrupt
 *	@li @c		SPICTLSPRIE			Receive data buffer full interrupt
 *
 * @retval	SPI_NG		Failed.
 * @retval	SPI_OK		Success.
 ***************************************************************************/
int EnableSPIInt( unsigned char intSelect )
{
	/// Check parameter
	if( (intSelect & 0xcf) != 0 ){
		return 0;
	}

	/// Set SPI interrupt enable
	SPI_CTL0 |= intSelect;

	return 1;
}

/****************************************************************************
 * DisableSPIInt function.
 *
 * @brief	SPI interrupt disable.
 *
 * @param	intSelect
 *	@li @c		SPICTLSPTIE			Transmit data buffer empty interrupt
 *	@li @c		SPICTLSPRIE			Receive data buffer full interrupt
 *
 * @retval	SPI_NG		Failed.
 * @retval	SPI_OK		Success.
 ***************************************************************************/
int DisableSPIInt( unsigned char intSelect )
{
	/// Check parameter
	if( (intSelect & 0xcf) != 0 ){
		return 0;
	}

	/// Set SPI interrupt disable
	SPI_CTL0 &=  ~((unsigned short)intSelect);

	return 1;
}

/****************************************************************************
 * InitSPIInt function.
 *
 * @brief	SPI interrupt initialize.
 *
 * @param	intLevel
 *	@li @c		SPI_INTLVL_0		SPI interrupt level 0
 *	@li @c		SPI_INTLVL_1		SPI interrupt level 1
 *	@li @c		SPI_INTLVL_2		SPI interrupt level 2
 *	@li @c		SPI_INTLVL_3		SPI interrupt level 3
 *	@li @c		SPI_INTLVL_4		SPI interrupt level 4
 *	@li @c		SPI_INTLVL_5		SPI interrupt level 5
 *	@li @c		SPI_INTLVL_6		SPI interrupt level 6
 *	@li @c		SPI_INTLVL_7		SPI interrupt level 7
 *
 * @retval	SPI_NG		Failed.
 * @retval	SPI_OK		Success.
 ***************************************************************************/
int InitSPIInt( unsigned char intLevel )
{
	/// Check parameter
	if( intLevel > 7 ){
		return 0;
	}

	/// Set UART interrupt level
	ITC_LV7 = intLevel;

	return 1;
}

/******************************************************************************************
 * isSPIInt function.
 *
 * @brief	Check SPI interrupt.
 *
 * @param	intType
 *	@li @c		SPISTSPTBE		Transmit data buffer empty flag                        [D0]
 *	@li @c		SPISTSPRBF		Receive data buffer full flag                          [D1]
 *	@li @c		SPISTSPBSY		Transfer busy flag(master) / ss signal low flag(slave) [D2]
 *
 * @retval	SPIINT_NOTOCCURRED		Not occurred.
 * @retval	SPIINT_OCCURRED			Occurred.
 ******************************************************************************************/
int isSPIInt( unsigned char intType )
{
	/// Check parameter
	if( (intType & 0xf8) != 0 ){
		return 0;
	}

	/// Check SPI interrupt
	if( (SPI_ST0 & intType) != 0 ){
		return 1;
	}

	return 0;
}



