/********************************************************************************
 *
 * @file	main.c
 * @brief	This is t16a2-sample program for S1C17656 demonstration.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 *******************************************************************************/
/********************************************************************************
 * @mainpage	S1C17656 T16A2
 *
 * @section		intro Introduction
 * Sample code gets timer counter value by advanced timer compare A or B interrupt.
 * The code outputs waveform to TOUTA0 or TOUTB0 terminal.
 * When wait T8 timer interrupt, the code changes Halt mode of CPU,
 * reduce consumption electricity.
 *
 * @subsection	step1	Step 1: Start T16A2 timer.
 *						Enable compare A or B interrupt.
 *						Start T16A2 timer interrupt.
 * @subsection	step2	Step 2: Get T16A2 timer count data.
 *						When occur T16A2 timer interrupt, keep T16A2 timer
 *						count data.
 * @subsection	step3	Step 3: Repeat five times.
 *						if compare B interrupt occurred five times,
 *						stop T16A2 timer.
 * @subsection	step4	Step 4: Display count data.
 *						Display count data and kind of interrupt
 *						to simulated I/O window.
 *						End sample program.
 *
 * @section		install Installation
 * Start the IDE and import the "s1c17656_t16a2" project.
 *
 *******************************************************************************/

/// #include
#include <string.h>
#include <stdio.h>
#include <c17656_reg.h>
#include <init.h>
#include <t16a2.h>

/// Prototype
int main(void);
void IntT16A2(void)__attribute__ ((interrupt_handler));

/// global
volatile int loopCount;
unsigned short getdata[10] = {0};
unsigned char dataName[10] = {0};

/****************************************************************************
 * Main function.
 *
 * @retval	1	Normality
 * @retval	0	Abnormality
 ***************************************************************************/
int main(void)
{
	/// End Status
	int rnf;
	int rnf_main = 1;

	/// Initial Parameter
	int i;
	T16A2_INIT_PARAM param;

	DebugModeMisc( 0 );			/// Misc Stop in Debug Mode
	DebugModePsc( 0 );			/// Prescaler Stop in Debug Mode
	SetClockGear( 0 );			/// Set Clock Gear *default=0x0
	ControlClg( 3 );			/// Run Clg

	printf( "<<< T16 PWM timer demonstration start >>>\n" );

	param.clkdiv = 0xe;			/// 1/16384
	param.clksrc = 0;			/// OSC3B
	param.hcm = 0;				/// normal mode
	param.cbufen = 0;			/// disable
	param.trmd = 0;				/// repeat mode
	param.captrg = 0;			/// none trigger
	param.toutmd = 0;			/// output off
	param.toutinv = 0;			/// normal mode
	param.ccmd = 0;				/// Comparator mode
	param.dataA	 = 2000000 / 16384 / 4;			/// compare A data
	param.dataB	 = 2000000 / 16384 / 2;			/// compare B data

	/// T16A2 Initialize
	rnf = InitT16A2( &param );
	if( rnf == 0 ){
		printf("***  InitT16A2 ERROR  ***\n");
		rnf_main = 0;
	}
	/// Compare x data initialize
	DataInitT16A2( &param );

	/// Set interrupt level 3
	rnf = InitT16A2Int( 3 );
	if( rnf == 0 ){
		printf("***  InitT16A2Int ERROR  ***\n");
		rnf_main = 0;
	}

	/// interrupt enable
	rnf = EnableT16A2Int( 0x01 | 0x02 );
	if( rnf == 0 ){
		printf("***  EnableT16A2Int ERROR  ***\n");
		rnf_main = 0;
	}

	/// T16A2 Start
	StartT16A2();

	loopCount = 0;
	while ( loopCount < 10 ){
		asm( "ei" );
		asm( "halt" );
		asm( "nop" );
		asm( "di" );
	}

	/// T16A2 timer Stop
	StopT16A2();

	/// T16A2 interrupt level 0
	rnf = InitT16A2Int( 0 );
	if( rnf == 0 ){
		printf("***  InitT16A2Int ERROR  ***\n");
		rnf_main = 0;
	}

	/// T16A2 interrupt disable
	rnf = DisableT16A2Int( 0x01 | 0x02 );
	if( rnf == 0 ){
		printf("***  DisableT16A2Int ERROR  ***\n");
		rnf_main = 0;
	}


	for ( i=0; i<loopCount; i++ ){
		printf( "*** T16 PWM compare %c Interrupt :%d***\n", dataName[i], getdata[i] );
	}

	printf( "<<< T16 PWM timer demonstration finish >>>\n" );

	return ( rnf_main );
}


/****************************************************************************
 * IntT16A2 function.
 *
 * @brief	 T16A2 interrupt routine.
 ***************************************************************************/
void IntT16A2(void)
{
	if( isT16A2Int(1)){
		T16A_IFLG0 = 0x01;
		getdata[loopCount] = GetT16A2Count( 1 );
		dataName[loopCount] = 'A';
		loopCount++;
	}

	if( isT16A2Int(2)){
		T16A_IFLG0 = 0x02;
		getdata[loopCount] = GetT16A2Count( 2 );
		dataName[loopCount] = 'B';
		loopCount++;
	}
}


