/****************************************************************************
 *
 * @file	t16a2.c
 * @brief	This is t16a2 driver program file for S1C17656.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 ***************************************************************************/

#include <c17656_reg.h>
#include <t16a2.h>

/****************************************************************************
 * InitT16A2 function.
 *
 * @brief	T16A2 timer initialize.
 *
 * @param	 *param
 *  @li  param->clkdiv
 *	   	@c 	T16A2_CLKDIV_1		1/1
 *   	@c 	T16A2_CLKDIV_2		1/2
 *		@c	T16A2_CLKDIV_4		1/4
 *	 	@c	T16A2_CLKDIV_8		1/8
 *	 	@c	T16A2_CLKDIV_16		1/16
 *	 	@c	T16A2_CLKDIV_32		1/32
 *	 	@c	T16A2_CLKDIV_64		1/64
 *	 	@c	T16A2_CLKDIV_128	1/128
 *	 	@c	T16A2_CLKDIV_256	1/256
 *	 	@c	T16A2_CLKDIV_512	1/512
 *	 	@c	T16A2_CLKDIV_F256	F256Hz( Only OSC1)
 *	 	@c	T16A2_CLKDIV_1024	1/1024
 *	 	@c	T16A2_CLKDIV_2048	1/2048
 *	 	@c	T16A2_CLKDIV_4096	1/4096
 *	 	@c	T16A2_CLKDIV_8192	1/8192
 *		@c	T16A2_CLKDIV_16384	1/16384
 * @li	param->clksrc
 *	 	@c	T16A2_CLKSRC_OSC3B	OSC3B
 *	 	@c	T16A2_CLKSRC_OSC1	OSC1
 * @li	param->hcm
 *	 	@c	NORMALCLK_MODE		Normal clock mode
 *	 	@c	HALFCLK_MODE		Half clock mode
 * @li	param->cbufen
 *	 	@c	T16A2_DISABLE		Disable
 *	 	@c	T16A2_ENABLE		Enable
 * @li	param->trmd
 *	 	@c	T16A2_REPEAT		Repeat mode
 *	 	@c	T16A2_ONESHOT		One-shot mode
 * @li	param->captrg
 *	 	@c	NONE_TRIGGER		None trigger
 *	 	@c	EDGE_UP				Up edge
 *	 	@c	EDGE_DOWN			Down edge
 *	 	@c	EDGE_UPANDDOWN		Up and down edge
 * @li	param->toutmd
 *	 	@c	OUTPUT_OFF			Output off
 *	 	@c	COMAUP_COMBDOWN		CompareA:up, CompareB:down
 *	 	@c	COMATGR_COMBNML		CompareA:toggle, CompareB:normal
 *	 	@c	COMANML_COMBTGR		CompareA:normal, CompareB:toggle
 * @li	param->toutinv
 *	 	@c	T16A2_NORMAL		Normal mode
 *	 	@c	T16A2_INVERT		invert mode
 * @li	param->ccmd
 *	 	@c	T16A2_COMPARATOR	Comparator mode
 *	 	@c	T16A2_CAPTURE		Capture mode
 *
 * @retval	T16A2_SET_NG	Failed.
 * @retval	T16A2_SET_OK	Success.
 ***************************************************************************/
int InitT16A2( T16A2_INIT_PARAM *param)
{
	/// Parameter Check
 	if( ((param->clkdiv) > 0xE)	||
		((param->clksrc) > 1)	||
 		((param->hcm) > 1 )		||
 		((param->cbufen) > 1 )	||
 		((param->captrg) > 3 )	||
 		((param->trmd) > 1 )	||
 		((param->toutmd) > 3 )	||
 		((param->toutinv) > 1 )	||
 		((param->ccmd) > 1 )	){
			return 0;
	}

	/// Stop Timer
	StopT16A2();

	/// Stop Clock
	T16A_CLK0 &= 0xfe;

	/// Set PORT Function in Advanced Timer
	P04_07PMUX = 0x05;

	/// Set Capture A/B Trigger
	/// Set TOUT A/B Mode
	/// Set TOUT A/B invert
	/// Set Mode
	T16A_CCCTL0 = ((param->captrg << 6) | (param->captrg << 14) |
				 (param->toutmd << 4)	| (param->toutmd << 12) |
				 (param->toutinv << 1)	| (param->toutinv << 9) |
				 (param->ccmd)			| (param->ccmd << 8));

#if 0
	/// Set Capture A/B Trigger
	T16A_CCCTL0_CAPATRG = param->captrg;
	T16A_CCCTL0_CAPBTRG = param->captrg;

	/// Set TOUT A/B Mode
	T16A_CCCTL0_TOUTAMD = param->toutmd;
	T16A_CCCTL0_TOUTBMD = param->toutmd;

	/// Set TOUT A/B invert
	T16A_CCCTL0_TOUTAINV = param->toutinv;
	T16A_CCCTL0_TOUTBINV = param->toutinv;

	/// Set Mode
	T16A_CCCTL0_CCAMD = param->ccmd;
	T16A_CCCTL0_CCBMD = param->ccmd;
#endif

	/// Set half clock mode
	/// Set compare buffer enable
	/// Set counter mode
	T16A_CTL0 = ((param->hcm << 6) | (param->cbufen << 3) | (param->trmd << 2));

#if 0
	/// Set half clock mode
	T16A_CTL0_HCM = param->hcm;

	/// Set compare buffer enable
	T16A_CTL0_CBUFEN = param->cbufen;

	/// Set counter mode
	T16A_CTL0_TRMD = param->trmd;
#endif

	/// Set Clock Source
	/// Set Clock Division Ratio
	/// Clock Enable
	T16A_CLK0 = ((param->clksrc << 2) | (param->clkdiv << 4) | 0x1);

#if 0
	/// Set Clock Source
	T16A_CLK0_T16ACLKSRC = param->clksrc;

	/// Set Clock Division Ratio
	T16A_CLK0_T16ACLKD = param->clkdiv;

	/// Clock Enable
	T16A_CLK0_T16ACLKE = T16A2_ENABLE;
#endif

	return 1;
}

/****************************************************************************
 * DataInitT16A2 function.
 *
 * @brief	 Set T16A2 PWM timer count value.
 *
 * @param	 *param
 *  @li	param->dataA
 *	 	@c	0x0000-0xffff	Compare A data
 *  @li	param->dataB
 *	 	@c	0x0000-0xffff	Compare B data
 ***************************************************************************/
void DataInitT16A2( T16A2_INIT_PARAM *param )
{
	/// Set Compare Data
	T16A_CCA0 = param->dataA;
	T16A_CCB0 = param->dataB;

	/// Timer Reset
	T16A_CTL0 |= 0x0002;
}

/****************************************************************************
 * GetT16A2Count function.
 *
 * @brief	 Get T16A2 timer counter value.
 *
 * @param	cntType
 *	@li @c		NORMAL_CNTDATA	Normal count data
 *	@li @c		COMA_DATA		Compare A data
 *	@li @c		COMB_DATA		Compare B data
 *
 * @return 	data	Return data of one.
 ***************************************************************************/
unsigned short GetT16A2Count( unsigned char cntType )
{
	unsigned short data = 0;

	/// Get Data
	switch( cntType ){
		case NORMAL_CNTDATA:
						data = T16A_TC0;
						break;
		case COMA_DATA:
						data = T16A_CCA0;
						break;
		case COMB_DATA:
						data = T16A_CCB0;
						break;
		default:
						break;
	}

	return ( data );
}

/****************************************************************************
 * StartT16A2 function.
 *
 * @brief	T16 timer start.
 ***************************************************************************/
void StartT16A2(void)
{
	/// Timer Start
	T16A_CTL0 |= 0x0001;
}

/****************************************************************************
 * StopT16A2 function.
 *
 * @brief	T16A2 timer stop.
 ***************************************************************************/
void StopT16A2(void)
{
	/// Timer Stop
	T16A_CTL0 &= 0xfffe;
}

/****************************************************************************
 * InitT16A2Int function.
 *
 * @brief	T16A2 timer interrupt initialize.
 *
 * @param	intLevel
 *	@li @c		T16A2_INTLV_0		Interrupt Level 0
 *	@li @c		T16A2_INTLV_1		Interrupt Level 1
 *	@li @c		T16A2_INTLV_2		Interrupt Level 2
 *	@li @c		T16A2_INTLV_3		Interrupt Level 3
 *	@li @c		T16A2_INTLV_4		Interrupt Level 4
 *	@li @c		T16A2_INTLV_5		Interrupt Level 5
 *	@li @c		T16A2_INTLV_6		Interrupt Level 6
 *	@li @c		T16A2_INTLV_7		Interrupt Level 7
 *
 * @retval	T16A2_SET_NG	Failed.
 * @retval	T16A2_SET_OK	Success.
 ***************************************************************************/
int InitT16A2Int( unsigned char intLevel )
{
	/// Parameter Check
	if( intLevel > 7 ){
		return 0;
	}

	/// T16A2 Interrupt Level Set
	ITC_LV3 &= 0x00ff;
	ITC_LV3 |= (unsigned short)intLevel << 8;

	return 1;
}

/****************************************************************************
 * EnableT16A2Int function.
 *
 * @brief	T16A2 timer interrupt enable.
 *
 * @param	intSelect
 *	@li @c		INT_COMA		Compare A Interrupt
 *	@li @c		INT_COMB		Compare B Interrupt
 *	@li @c		INT_CAPA		Capture A Interrupt
 *	@li @c		INT_CAPB		Capture B Interrupt
 *	@li @c		INT_CAPAOVE		Capture A OverWrite Interrupt
 *	@li @c		INT_CAPBOVE		Capture B OverWrite Interrupt
 *
 * @retval	T16A2_SET_NG	Failed.
 * @retval	T16A2_SET_OK	Success.
 ***************************************************************************/
int EnableT16A2Int( unsigned short intSelect )
{
	/// Check parameter
	if( intSelect > 0x3f ){
		return 0;
	}

	/// T16A2 Interrupt Reset
	ClrT16A2IntFlg( intSelect );

	/// T16A2 Interrupt Enable
	T16A_IEN0 = intSelect;

	return 1;
}

/****************************************************************************
 * DisableT16A2Int function.
 *
 * @brief	T16A2 timer interrupt disable.
 *
 * @param	intSelect
 *	@li @c		INT_COMA		Compare A Interrupt
 *	@li @c		INT_COMB		Compare B Interrupt
 *	@li @c		INT_CAPA		Capture A Interrupt
 *	@li @c		INT_CAPB		Capture B Interrupt
 *	@li @c		INT_CAPAOVE		Capture A OverWrite Interrupt
 *	@li @c		INT_CAPBOVE		Capture B OverWrite Interrupt
 *
 * @retval	T16A2_SET_NG	Failed.
 * @retval	T16A2_SET_OK	Success.
 ***************************************************************************/
int DisableT16A2Int( unsigned short intSelect )
{
	/// Check parameter
	if( intSelect > 0x3f ){
		return 0;
	}

	/// T16A2 Interrupt Disable
	T16A_IEN0 &= ~(intSelect);

	/// T16A2 Interrupt Reset
	ClrT16A2IntFlg( intSelect );

	return 1;
}

/****************************************************************************
 * isT16A2Int function.
 *
 * @brief	 Check T16A2 timer interrupt flag.
 *
 * @param	cntType
 *	@li @c		NORMAL_CNTDATA	Normal count data
 *	@li @c		COMA_DATA		Compare A data
 *	@li @c		COMB_DATA		Compare B data
 *
 * @retval	PORT_NOTOCCURRED	Not occurred
 * @retval	PORT_OCCURRED		Occurred
 ***************************************************************************/
int isT16A2Int( unsigned char cntType )
{
	if( cntType == 1 ){
		if((T16A_IFLG0 & 0x0001) == 1 ){
			return 1;
		}
	} else if( cntType == 2 ){
		if((T16A_IFLG0 & 0x0002) == 2 ){
			return 1;
		}
	} else {
		;
	}

	return 0;
}

/****************************************************************************
 * ClrT16A2IntFlg function.
 *
 * @brief	Clear T16A2 timer interrupt flag.
 *
 * @param	intSelect
 *	@li @c		INT_COMA		Compare A Interrupt
 *	@li @c		INT_COMB		Compare B Interrupt
 *	@li @c		INT_CAPA		Capture A Interrupt
 *	@li @c		INT_CAPB		Capture B Interrupt
 *	@li @c		INT_CAPAOVE		Capture A OverWrite Interrupt
 *	@li @c		INT_CAPBOVE		Capture B OverWrite Interrupt
 *
 ***************************************************************************/
void ClrT16A2IntFlg( unsigned short intSelect )
{
	/// T16A2 Interrupt Reset
	T16A_IFLG0 = intSelect;
}



