/****************************************************************************
 *
 * @file	main.c
 * @brief	This is t8-sample program for S1C17656 demonstration.
 *
 * @par		Copyright:
 * 			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 * 			- 2014/08/20	1.00
 *				- First revision
 *
 ***************************************************************************/
/****************************************************************************
 * @mainpage	S1C17656 T8
 *
 * @section		intro Introduction
 * Sample code gets counter value of timer by T8 timer interrupt.
 * When wait T8 timer interrupt, the code changes Halt mode of CPU,
 * reduce consumption electricity.
 *
 * @subsection	step1	Step 1: Start T8 timer interrupt.
 *						Start T8 timer interrupt.
 *						Change Halt mode of CPU.
 * @subsection	step2	Step 2: Get T8 timer count data.
 *						When occur T8 timer interrupt, CPU release Halt mode.
 *						Keep T8 timer count data to internal variable.
 *						Change Halt mode of CPU.
 * @subsection	step3	Step 3: Repeat ten times.
 *						Repeat T8 timer interrupt ten times.
 * @subsection	step4	Step 4: Display count data.
 *						Stop T8 timer interrupt.
 *						Display count data when occurred to simulated I/O window.
 *						End sample program.
 *
 * @section		install Installation
 * Start the IDE and import the "s1c17656_t8" project.
 *
 ***************************************************************************/
/// #include
#include <string.h>
#include <stdio.h>
#include <c17656_reg.h>
#include <init.h>
#include <t8.h>

/// Prototype
int main(void);
void IntT8(void)__attribute__ ((interrupt_handler));

/// global
volatile int loopCount;
volatile unsigned char t8fFlg;
unsigned short T8GetCountData[10] = {0};

/****************************************************************************
 * Main function.
 *
 * @retval	1	Normality
 * @retval	0	Abnormality
 ***************************************************************************/
int main(void)
{
	int i;
	int rnf;
	int rnf_main = 1;

	DebugModeMisc( 0 );			/// Misc Stop in Debug Mode
	DebugModePsc( 0 );			/// Prescaler Stop in Debug Mode
	SetClockGear( 0 );			/// Set Clock Gear *default=0x0
	ControlClg( 3 );			/// Run Clg

	printf( "<<< T8 timer demonstration start >>>\n" );

	/// T8F Initialize
	rnf = InitT8( 0, 7, 0x80 );
	if( rnf == 0 ){
		printf("***  InitT8 ERROR  ***\n");
		rnf_main = 0;
	}

	/// Interrupt Initialize
	rnf = InitT8Int( 3 );
	if( rnf == 0 ){
		printf("***  InitT8Int ERROR  ***\n");
		rnf_main = 0;
	}
	/// Interrupt enable
	EnableT8Int();

	/// Timer Start
	StartT8();


	t8fFlg		= 0;
	loopCount	= 0;
	while ( loopCount != 10 ) {
		asm ( "ei" );
		asm ( "halt" );
		asm ( "nop" );
		asm ( "di" );

		if ( t8fFlg==1 ){
			loopCount++;
			t8fFlg = 0;
		}
	}

	/// Timer Stop
	StopT8();

	/// Interrupt Initialize
	rnf = InitT8Int( 0 );
	if( rnf == 0 ){
		printf("***  InitT8Int ERROR  ***\n");
		rnf_main = 0;
	}
	/// Interrupt disable
	DisableT8Int();

	for ( i = 0; i < loopCount; i++ ) {
		printf( "*** T8 Interrupt %d time , count data at this time :%d***\n", i+1, T8GetCountData[i] );
	}

	printf( "<<< T8 demonstration finish >>>\n");

	return ( rnf_main );
}

/****************************************************************************
 * IntT8 function.
 *
 * @brief	 T8 interrupt routine.
 ***************************************************************************/
void IntT8(void)
{
	if ( T8_INT0_T8IF ){
		if( isT8Int() ){
			/// Interrupt Flag Reset
			ClrT8IntFlg();
			/// Count Data Set
			T8GetCountData[loopCount] = GetT8Count();

			t8fFlg = 1;
		}
	}
}


