/****************************************************************************
 *
 * @file	ct.c
 * @brief	This is ct driver program file for S1C17656.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 ***************************************************************************/

#include <c17656_reg.h>
#include <ct.h>

/****************************************************************************
 * ResetCTCount function.
 *
 * @brief	 Reset CT.
 ***************************************************************************/
void ResetCTCount( void )
{
	/// Clock timer reset
	CT_CTL |= 0x10;
}

/****************************************************************************
 * StartCT function.
 *
 * @brief	 Start CT.
 ***************************************************************************/
void StartCT(void)
{
	/// Clock timer start
	CT_CTL = 0x01;
}

/****************************************************************************
 * StopCT function.
 *
 * @brief	 Stop CT.
 ***************************************************************************/
void StopCT(void)
{
	/// Clock timer stop
	CT_CTL = 0x00;
}

/****************************************************************************
 * InitCTInt function.
 *
 * @brief	 Set Interrupt level.
 *
 * @param	intLevel
 *	@li @c		CT_INTLV_0		Interrupt Level 0
 *	@li @c		CT_INTLV_1		Interrupt Level 1
 *	@li @c		CT_INTLV_2		Interrupt Level 2
 *	@li @c		CT_INTLV_3		Interrupt Level 3
 *	@li @c		CT_INTLV_4		Interrupt Level 4
 *	@li @c		CT_INTLV_5		Interrupt Level 5
 *	@li @c		CT_INTLV_6		Interrupt Level 6
 *	@li @c		CT_INTLV_7		Interrupt Level 7
 *
 * @retval	CT_SET_NG	Failed.
 * @retval	CT_SET_OK	Success.
 ***************************************************************************/
int InitCTInt(unsigned char intLevel)
{
	/// Check parameter
	if( intLevel > 7 ){
		return 0;
	}

	/// Set interrupt level
	ITC_LV1 = (unsigned short)intLevel << 8;

	return 1;
}

/****************************************************************************
 * EnableCTInt function.
 *
 * @brief	 CT interrupt enable.
 *
 * @param	intSelect
 *	@li @c		CT_INT_1HZ		CT interrupt 1Hz
 *	@li @c		CT_INT_2HZ		CT interrupt 2Hz
 *	@li @c		CT_INT_8HZ		CT interrupt 8Hz
 *	@li @c		CT_INT_32HZ		CT interrupt 32Hz
 *
 * @return	rnf_sub		0: Failed, 1: Success
 ***************************************************************************/
int EnableCTInt(unsigned char intSelect)
{
	int rnf;
	int rnf_sub = 1;

	/// Check parameter
	if( intSelect > 0x0F ){
		rnf_sub = 0;
	}

	/// Interrupt clear
	rnf = ClrCTIntFlg(intSelect);
	if( rnf == 0 ){
		rnf_sub = 0;
	}

	/// Interrupt enable
	CT_IMSK = intSelect;

	return (rnf_sub);
}

/****************************************************************************
 * DisableCTInt function.
 *
 * @brief	 CT interrupt disable.
 *
 * @param	intSelect
 *	@li @c		CT_INT_1HZ		CT interrupt 1Hz
 *	@li @c		CT_INT_2HZ		CT interrupt 2Hz
 *	@li @c		CT_INT_8HZ		CT interrupt 8Hz
 *	@li @c		CT_INT_32HZ		CT interrupt 32Hz
 *
 * @return	rnf_sub		0: Failed, 1: Success
 ****************************************************************************/
int DisableCTInt(unsigned char intSelect)
{
	int rnf;
	int rnf_sub = 1;

	/// Check parameter
	if( intSelect > 0x0F ){
		rnf_sub = 0;
	}

	/// Interrupt disable
	CT_IMSK &= ~(intSelect);

	/// Interrupt clear
	rnf = ClrCTIntFlg(intSelect);
	if( rnf == 0 ){
		rnf_sub = 0;
	}

	return (rnf_sub);
}

/****************************************************************************
 * isCTInt function.
 *
 * @brief	Check CT interrupt.
 *
 * @param	intType
 *	@li @c		CT_INT_1HZ		CT interrupt 1Hz
 *	@li @c		CT_INT_2HZ		CT interrupt 2Hz
 *	@li @c		CT_INT_8HZ		CT interrupt 8Hz
 *	@li @c		CT_INT_32HZ		CT interrupt 32Hz
 *
 * @retval	CT_NOTOCCURRED	Not occurred.
 * @retval	CT_OCCURRED		Occurred.
 ***************************************************************************/
int isCTInt(unsigned char intType)
{
	/// Check parameter
	if( intType > 0x0F ){
		return 0;
	}

	/// Check CT interrupt occurred
	if( (CT_IFLG & intType) == intType ){
		return 1;
	}

	return 0;
}

/****************************************************************************
 * ClrCTIntFlg function.
 *
 * @brief	Clear CT interrupt flag.
 *
 * @param	intType
 *	@li @c		CT_INT_1HZ		CT interrupt 1Hz
 *	@li @c		CT_INT_2HZ		CT interrupt 2Hz
 *	@li @c		CT_INT_8HZ		CT interrupt 8Hz
 *	@li @c		CT_INT_32HZ		CT interrupt 32Hz
 *
 * @retval	CT_SET_NG		Failed.
 * @retval	CT_SET_OK		Success.
 ***************************************************************************/
int ClrCTIntFlg(unsigned char intType)
{
	/// Check parameter
	if( intType > 0x0F ){
		return 0;
	}

	CT_IFLG = intType;

	return 1;
}


