/****************************************************************************
 *
 * @file	uart.c
 * @brief	This is uart driver program file for S1C17656.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 ***************************************************************************/

// #include
#include <c17656_reg.h>
#include <uart.h>

/****************************************************************************
 * InitUART function.
 *
 * @brief	UART initialize.
 *
 * @param	 *param
 *  @li  param->dataLen
 *	   	@c 	UARTMODCHLN_7BITS		7Bits
 *   	@c 	UARTMODCHLN_8BITS		8Bits
 *  @li	param->parity
 *	 	@c	UARTPARTY_NOPARTY		No parity/Even parity
 *	 	@c	UARTPARTY_ODDPARTY		With parity/Odd parity
 *	 	@c	UARTPARTY_EVENPARTY		With parity/Even parity
 *  @li	param->stopBit
 *	 	@c	UARTMODSTPB_1BIT		1Bit
 *	 	@c	UARTMODSTPB_2BITS		2Bits
 *  @li	param->recByte
 *	 	@c	UARTCTLRBFI_1BYTE		1Byte
 *	 	@c	UARTCTLRBFI_2BYTES		2Bytes
 *  @li	param->clock
 *	 	@c	UARTSRC_OSC3B			OSC3B
 *	 	@c	UARTSRC_OSC1			OSC1
 *	 	@c	UARTSRC_EXTRNLCLK		EXternal clock
 *  @li	param->div
 *	 	@c	UARTCLKDIV_1			1/1
 *	 	@c	UARTCLKDIV_2			1/2
 *	 	@c	UARTCLKDIV_4			1/4
 *	 	@c	UARTCLKDIV_8			1/8
 *  @li	param->brr
 *	 	@c	0x00-0xff				Baud Rate Count
 *  @li	param->fmd
 *	 	@c	0x0-0xf					Fine Mode
 *
 * @retval	UART_NG		Failed.
 * @retval	UART_OK		Success.
 ***************************************************************************/
int InitUART( UART_INIT_PARAM *param )
{
	/// Check parameter
	if( (param->dataLen) > 1	||
		(param->parity) > 2		||
		(param->stopBit) > 1	||
		(param->recByte) > 1	||
		(param->clock) > 3		||
		(param->div) > 3		||
		(((param->fmd) & 0xf0) != 0 )){
			return 0;
	}

	/// UART disable bit set
	UART_CTL0 &= 0xfe;

	/// UART clock disable set
	UART_CLK0 &= 0xfe;

	/// UART port function set
	P00_03PMUX = 0x15;

	/// Clock source and division ratio set
	UART_CLK0 = ((param->clock << 2) | (param->div << 4));
	//UART_CLK0_UTCLKSRC = param->clock;
	//UART_CLK0_UTCLKD = param->div;

	/// Baud rate count set
	UART_BR0 = param->brr;

	/// Fine mode set
	UART_FMD0 = param->fmd;

	/// Character length set
	/// Stop bit set
	UART_MOD0 = (UART_MOD0 & 0xed) | ((param->dataLen << 4) | (param->stopBit << 1));

//	UART_MOD0_CHLN = param->dataLen;
//	UART_MOD0_STPB = param->stopBit;

	/// Parity mode set
	switch( param->parity ){
		case 0:
				UART_MOD0 = (UART_MOD0 & 0xf3);
				//UART_MOD0_PREN = UARTMODPREN_NOPARITY;
				//UART_MOD0_PMD = UARTMODPMD_EVEN;
				break;
		case 1:
				UART_MOD0 = (UART_MOD0 & 0xf3) | 0x0c;
				//UART_MOD0_PREN = UARTMODPREN_WITHPARITY;
				//UART_MOD0_PMD = UARTMODPMD_ODD;
				break;
		case 2:
				UART_MOD0 = (UART_MOD0 & 0xf3) | 0x08;
				//UART_MOD0_PREN = UARTMODPREN_WITHPARITY;
				//UART_MOD0_PMD = UARTMODPMD_EVEN;
				break;
		default:
				break;
	}


	/// Receive buffer full interrupt enable/disable set
	UART_CTL0 = (UART_CTL0 & 0xfd) | (param->recByte << 1);

	return 1;
}

/****************************************************************************
 * SendDataUART function.
 *
 * @brief	UART transmit data write.
 *
 * @param	data
 *	@li @c		0x00-0xff(0x00-0x7f)	transmit data(7 bits mode)
 ***************************************************************************/
void SendDataUART( unsigned char data )
{
	/// UART transmit data write
	UART_TXD0 = data;
}

/****************************************************************************
 * ReceiveDataUART function.
 *
 * @brief	UART receive data read.
 *
 * @return	data	UART receive data.
 ***************************************************************************/
unsigned char ReceiveDataUART(void)
{
	/// UART receive data read
	return ( UART_RXD0 );
}

/****************************************************************************
 * StartUART function.
 *
 * @brief	Set UART enable.
 ***************************************************************************/
void StartUART(void)
{
	/// Start UART clock
	UART_CLK0 |= 0x01;

	/// Set UART enable
	UART_CTL0 |= 0x01;
}

/****************************************************************************
 * StopUART function.
 *
 * @brief	Set UART enable.
 ***************************************************************************/
void StopUART(void)
{
	/// Set UART disable
	UART_CTL0 &= 0xfe;

	/// Stop UART clock
	UART_CLK0 &= 0xfe;
}

/****************************************************************************
 * EnableUARTInt function.
 *
 * @brief	UART interrupt enable.
 *
 * @param	intSelect
 *	@li @c		UARTINT_TIEN		Transmit buffer empty interrupt
 *	@li @c		UARTINT_RIEN		Receive buffer full interrupt
 *	@li @c		UARTINT_REIEN		Receive error interrupt
 *	@li @c		UARTINT_TEIEN		End of transmission interrupt
 *
 * @retval	UART_NG		Failed.
 * @retval	UART_OK		Success.
 ***************************************************************************/
int EnableUARTInt( unsigned char intSelect )
{
	/// Check parameter
	if( (intSelect & 0x0f) != 0 ){
		return 0;
	}

	/// Set UART interrupt enable
	UART_CTL0 |= intSelect;

	return 1;
}

/****************************************************************************
 * DisableUARTInt function.
 *
 * @brief	UART interrupt disable.
 *
 * @param	intSelect
 *	@li @c		UARTINT_TIEN		Transmit buffer empty interrupt
 *	@li @c		UARTINT_RIEN		Receive buffer full interrupt
 *	@li @c		UARTINT_REIEN		Receive error interrupt
 *	@li @c		UARTINT_TEIEN		End of transmission interrupt
 *
 * @retval	UART_NG		Failed.
 * @retval	UART_OK		Success.
 ***************************************************************************/
int DisableUARTInt( unsigned char intSelect )
{
	/// Check parameter
	if( (intSelect & 0x0f) != 0 ){
		return 0;
	}

	/// Set UART interrupt disable
	UART_CTL0 &= ~(intSelect);

	return 1;
}

/****************************************************************************
 * InitUARTInt function.
 *
 * @brief	UART interrupt initialize.
 *
 * @param	intLevel
 *	@li @c		UART_INTLVL_0		UART interrupt level 0
 *	@li @c		UART_INTLVL_1		UART interrupt level 1
 *	@li @c		UART_INTLVL_2		UART interrupt level 2
 *	@li @c		UART_INTLVL_3		UART interrupt level 3
 *	@li @c		UART_INTLVL_4		UART interrupt level 4
 *	@li @c		UART_INTLVL_5		UART interrupt level 5
 *	@li @c		UART_INTLVL_6		UART interrupt level 6
 *	@li @c		UART_INTLVL_7		UART interrupt level 7
 *
 * @retval	UART_NG		Failed.
 * @retval	UART_OK		Success.
 ***************************************************************************/
int InitUARTInt( unsigned char intLevel )
{
	/// Check parameter
	if( intLevel > 7 ){
		return 0;
	}

	/// Set UART interrupt level
	ITC_LV6 = intLevel;

	return 1;
}

/****************************************************************************
 * isUARTInt function.
 *
 * @brief	Check UART interrupt.
 *
 * @param	intType
 *	@li @c		UARTSTTDBE		Transmit data buffer empty flag   [D0]
 *	@li @c		UARTSTRDRY		Receive data ready flag           [D1]
 *	@li @c		UARTSTTRBS		Transmit busy flag                [D2]
 *	@li @c		UARTSTRD2B		Second byte receive flag          [D3]
 *	@li @c		UARTSTOER		Overrun error flag                [D4]
 *	@li @c		UARTSTPER		Parity error flag                 [D5]
 *	@li @c		UARTSTFER		Framing error flag                [D6]
 *	@li @c		UARTSTTRE		End of transmission completed flag[D7]
 *
 * @retval	UARTINT_NOTOCCURRED		Not occurred.
 * @retval	UARTINT_OCCURRED		Occurred.
 ***************************************************************************/
int isUARTInt( unsigned char intType )
{
	/// Check UART interrupt
	if( (UART_ST0 & intType) != 0 ){
		return 1;
	}

	return 0;
}

/****************************************************************************
 * ClrUARTIntFlg function.
 *
 * @brief	Clear UART interrupt flag.
 *
 * @param	intType
 *	@li @c		UARTSTOER		Overrun error flag                [D4]
 *	@li @c		UARTSTPER		Parity error flag                 [D5]
 *	@li @c		UARTSTFER		Framing error flag                [D6]
 *	@li @c		UARTSTTRE		End of transmission completed flag[D7]
 *
 * @retval	UART_NG		Failed.
 * @retval	UART_OK		Success.
 ***************************************************************************/
int ClrUARTIntFlg( unsigned char intType )
{
	/// Check parameter
	if( (intType & 0x0f) != 0 ){
		return 0;
	}

	/// Clear interrupt flag
	UART_ST0 = intType;

	return 1;
}


