/*******************************************************************************
 *
 * @file	boot.c
 * @brief	This is vector and interrupt program with C.
 *
 * @par		Copyright:
 * 			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 * 			- 2014/08/20
 *
 ******************************************************************************/

/* #include */
#include <string.h>
#include <stdio.h>
#include <libstdio.h>
#include <init.h>

/* Prototype */
void boot( void );
void ClearBss( void );
void CopyLmaToVma( void );
void end(void);
void dummy(void)__attribute__ ((interrupt_handler));
//void nmi(void)__attribute__ ((interrupt_handler));
void emu_copro_process(void)__attribute__ ((interrupt_handler));

/* extern */
extern int main(void);
extern void IntT8(void)__attribute__ ((interrupt_handler));
extern void nmi(void)__attribute__ ((interrupt_handler));

extern unsigned long __START_bss;
extern unsigned long __END_bss;
extern unsigned long __START_data;
extern unsigned long __START_data_lma;
extern unsigned long __END_data;
extern unsigned long __START_simio;
extern unsigned long __START_simio_lma;
extern unsigned long __END_simio;

/* special definitions for vector table  */
typedef void func(void);
#ifdef __POINTER16
#define VECTOR(vec)  vec,0
#else
#define VECTOR(vec)  (vec)
#endif

/* vector table */
func *const vector[] = {		// No	Base+	Name
	VECTOR(boot),				// 0	00		Reset
	VECTOR(dummy),				// 1	04		Illegal Adress Int.
	VECTOR(nmi),				// 2	08		NMI
	VECTOR(emu_copro_process),	// 3	0c		copro precess
	VECTOR(0),					// 4	10		P0X Int.
	VECTOR(0),					// 5	14		reserved
	VECTOR(0),					// 6	18		reserved
	VECTOR(0),					// 7	1c		CT Int.
	VECTOR(0),					// 8	20		RTC Int.
	VECTOR(0),					// 9	24		reserved
	VECTOR(0),					// 10	28		LCD Int.
	VECTOR(0),					// 11	2c		T16A PWM Ch.0 Int.
	VECTOR(0),					// 12	30		reserved
	VECTOR(0),					// 13	34		reserved
	VECTOR(IntT8),				// 14	38		T8 Ch.0 Int.
	VECTOR(0),					// 15	3c		reserved
	VECTOR(0),					// 16	40		UART Ch.0 Int.
	VECTOR(0),					// 17	44		reserved
	VECTOR(0),					// 18	48		SPI Ch.0 Int.
	VECTOR(0),					// 19	4c		reserved
	VECTOR(0),					// 20	50		reserved
	VECTOR(0),					// 21	54		reserved
	VECTOR(0),					// 22	58		reserved
	VECTOR(0),					// 23	5c		reserved
	VECTOR(0),					// 24	60		reserved
	VECTOR(0),					// 25	64		reserved
	VECTOR(0),					// 26	68		reserved
	VECTOR(0),					// 27	6c		reserved
	VECTOR(0),					// 28	70		reserved
	VECTOR(0),					// 29	74		reserved
	VECTOR(0),					// 30	78		reserved
	VECTOR(0)					// 31	7c		reserved
};


/****************************************************************************
 * Boot function.
 *
 * @brief	First, this function turns off the "DSI noise filter".
 * 			Next, it resets the Stack Pointer, clear the bss area,
 * 			and copy LMA to VMA.
 *			Then, it calls "main" function and calls "end" function.
 ***************************************************************************/
void boot(void)
{
	int rnf;

	asm ( "xld.a  %sp, 0x07c0" );  	// Set SP in RAM

	ClearBss();						// clear the bss area
	CopyLmaToVma();					// copy LMA data to VMA data
    _init_sys();					// Initialize for sys.c
    _init_lib();					// Initialize for lib.c

	SetFlashcAccessCycle( 0 );

	asm ( "ei" );					//interrupt enable

	rnf = main(); 					// Call main
	if ( rnf == 0 ) {
		printf( "*** TEST ERROR ***\n" );
	} else {
		printf( "*** TEST OK ***\n" );
	}
	end(); 							// Call end
}


/****************************************************************************
 * ClearBss function.
 *
 * @brief	This function clears the bss area.
 ***************************************************************************/
void ClearBss(void)
{
	unsigned long ulCnt;
	unsigned char *pcData;

	pcData = (unsigned char *)&__START_bss;
	for ( ulCnt = 0; ulCnt < ((unsigned char *)&__END_bss - (unsigned char *)&__START_bss); ulCnt++ )
	{
		*pcData = 0;
		pcData++;
	}
}


/****************************************************************************
 * CopyLmaToVma function.
 *
 * @brief	This function copys LMA data to VMA data.
 ***************************************************************************/
void CopyLmaToVma(void)
{
	memcpy ((unsigned char *)&__START_data,
			(unsigned char *)&__START_data_lma,
			((unsigned char *)&__END_data - (unsigned char *)&__START_data) );
	memcpy ((unsigned char *)&__START_simio,
			(unsigned char *)&__START_simio_lma,
			((unsigned char *)&__END_simio - (unsigned char *)&__START_simio) );
}


/****************************************************************************
 * End function.
 *
 * @brief	This function has only infinite loop.
 ***************************************************************************/
void end(void)
{
	// infinite loop
	while ( 1 ){
		;
	}
}

/****************************************************************************
 * Dummy Interrupt function.
 *
 * @brief	This function has only infinite loop.
 ***************************************************************************/
void dummy(void)
{
	// infinite loop
	while ( 1 ){
		;
	}
}

