/****************************************************************************
 *
 * @file	clg.c
 * @brief	This is clg(osc) driver program file for S1C17656.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 ***************************************************************************/

#include <c17656_reg.h>
#include <clg.h>

int  StopOSC1( void );
int  StartOSC1( void );
int  StopOSC3B( void );
int  StartOSC3B( unsigned char osc3Frq );
int  SetWaitCycleClg( unsigned char oscType , unsigned char waitCycle );
int  ChgOSC( unsigned char chgOscType );

/****************************************************************************
 * StopOSC1 function.
 *
 * @brief	Stop OSC1.
 *
 * @retval	CLG_STOP_NG		Failed.
 * @retval	CLG_STOP_OK		Success.
 ***************************************************************************/
int StopOSC1(void)
{
	/// Check Clock Source
	if ( CLG_SRC_CLKSRC == 1 ){
		return 0;
	}

	/// STOP OSC1
	CLG_CTL &= 0xfd;

	return 1;
}

/****************************************************************************
 * StartOSC1 function.
 *
 * @brief	Start OSC1.
 *
 * @retval	CLG_STOP_NG		Failed.
 * @retval	CLG_STOP_OK		Success.
 ***************************************************************************/
int StartOSC1(void)
{

	/// START OSC1
	CLG_CTL |= 0x02;

	return 1;
}

/****************************************************************************
 * StopOSC3B function.
 *
 * @brief	Stop OSC3B.
 *
 * @retval	CLG_STOP_NG		Failed.
 * @retval	CLG_STOP_OK		Success.
 ***************************************************************************/
int StopOSC3B(void)
{
	/// Check CLKSRC
	if ( CLG_SRC_CLKSRC == 0 ) {
		return 0;
	}

	/// STOP OSC3B
	CLG_CTL &= 0xfb;

	return 1;
}

/****************************************************************************
 * StartOSC3B function.
 *
 * @brief	Start OSC3B.
 *
 * @param	osc3Frq
 *	@li @c		OSC3B_FRQ_4M		4MHz
 *	@li @c		OSC3B_FRQ_2M		2MHz
 *	@li @c		OSC3B_FRQ_1M		1MHz
 *	@li @c		OSC3B_FRQ_500K		500kHz
 *
 * @retval	CLG_STOP_NG		Failed.
 * @retval	CLG_STOP_OK		Success.
 ***************************************************************************/
int StartOSC3B(unsigned char osc3Frq)
{

	/// Check Parameter
	if ( osc3Frq > 3 ) {
		return 0;
	}

	/// OSC3B Frequency Select
	CLG_SRC &= 0x3f;
	CLG_SRC |= (osc3Frq << 6);

	/// START OSC3B
	CLG_CTL |= 0x04;

	return 1;
}

/****************************************************************************
 * SetWaitCycleClg function.
 *
 * @brief	Setup Wait Cycle for oscType.
 *
 * @param	oscType
 *	@li @c		OSC_WAITSRC_OSC1	OSC1
 *	@li @c		OSC_WAITSRC_OSC3B	OSC3B
 * @param	waitCycle
 *	@li @c		WAIT_CYCLE_00		16384/64 	cycles(OSC1/ OSC3B)
 *	@li @c		WAIT_CYCLE_01		reserved/32 cycles(OSC1/ OSC3B)
 *	@li @c		WAIT_CYCLE_02		4096/16 	cycles(OSC1/ OSC3B)
 *	@li @c		WAIT_CYCLE_03		reserved/8/	cycles(OSC1/ OSC3B)
 *
 * @retval	CLG_STOP_NG		Failed.
 * @retval	CLG_STOP_OK		Success.
 ***************************************************************************/
int SetWaitCycleClg(unsigned char oscType , unsigned char waitCycle)
{
	/// Check Parameter
	if( oscType > 3 ||
		waitCycle > 3) {
			return 0;
	}

	if( oscType == 1 ||
		 oscType == 2) {
			return 0;
	}

	if( oscType == 0){
		if(waitCycle == 1 ||
			waitCycle == 3	){
			return 0;
		}
	}

	/// Setup Wait Cycle for oscType
	switch( oscType ){
		case 0:
					CLG_WAIT &= 0xfc;
					CLG_WAIT |= waitCycle;
					break;
		case 3:
					CLG_WAIT &= 0x3f;
					CLG_WAIT |= (waitCycle << 6);
					break;
		default:
					break;
	}

	return 1;
}


/****************************************************************************
 * ChgOSC function.
 *
 * @brief	Change OSC Clock Source.
 *
 * @param	chgOscType
 *	@li @c		CHG_OSC3B_OSC1	Change to OSC1
 *	@li @c		CHG_OSC1_OSC3B	Change to OSC3B
 *
 * @retval	CLG_STOP_NG		Failed.
 * @retval	CLG_STOP_OK		Success.
 ***************************************************************************/
int ChgOSC(unsigned char chgOscType)
{
	/// Check Parameter
	if( chgOscType > 3 ) {
		return 0;
	}

	/// Change Clock Source
	switch( chgOscType ){
		case 2:
					CLG_SRC &= 0xfc;
					CLG_SRC |= 0x01;
					break;
		case 3:
					CLG_SRC &= 0xfc;
					break;
		default:
					break;
	}

	return 1;
}

