/****************************************************************************
 *
 * @file	main.c
 * @brief	This is wdt-sample program for S1C17656 demonstration.
 *
 * @par		Copyright:
 *			Copyright(C) SEIKO EPSON CORPORATION 2014-2014. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 2014/08/20	1.00
 *				- First revision
 *
 ***************************************************************************/
/****************************************************************************
 * @mainpage	S1C17656 Watchdog Timer (WDT)
 *
 * @section		intro Introduction
 * Sample code confirm that Watchdog timer was cleared.
 * The code occur NMI interrupt by Watchdog timer.
 *
 * @subsection	step1	Step 1: Start T8 and WDT.
 *						Start T8 timer and WDT.
 * @subsection	step2	Step 2: WDT clear.
 * 						Clear WDT when T8 timer interrupt occurred.
 * @subsection	step3	Step 3: Stop T8.
 *						T8 timer interrupt occurred ten times, stop T8 timer.
 * @subsection	step4	Step 4: Display message.
 *						Display message to simulated I/O window,
 *						when NMI interrupt occurred by WDT.
 *						End sample program.
 *
 * @section		install Installation
 * Start the IDE and import the "s1c17656_wdt" project.
 *
 ***************************************************************************/

/// #include
#include <string.h>
#include <stdio.h>
#include <c17656_reg.h>
#include <init.h>
#include <clg.h>
#include <t8.h>
#include <wdt.h>

/// Prototype
int main(void);
void IntT8(void)__attribute__ ((interrupt_handler));
void nmi(void)__attribute__ ((interrupt_handler));

#define WDT_T8_LOOP_CT_MAX	10		///< Loop counter MAX

/// global
volatile unsigned int t8Flag = 0;
volatile unsigned int nmiFlag = 0;

/****************************************************************************
 * Main function.
 *
 * @retval	1	Normality
 * @retval	0	Abnormality
 ***************************************************************************/
int main(void)
{
	int T8LoopCount;
	int rnf;
	int rnf_main = 1;

	DebugModeMisc( 0 );			/// Misc Stop in Debug Mode
	DebugModePsc( 0 );			/// Prescaler Stop in Debug Mode
	SetClockGear( 0 );			/// Set Clock Gear *default=0x0
	ControlClg( 3 );			/// Run Clg

	/// OSC1A Wait Cycle
	rnf = SetWaitCycleClg( 0, 0 );
	if ( rnf == 0 ) {
		printf("***  SetWaitCycleClg ERROR  ***\n");
		rnf_main = 0;
	}
	/// wait for the start and stability of OSC1.
	rnf = StartOSC1();
	if( rnf == 0 ){
		printf("***  StartOSC1 ERROR  ***\n");
		rnf_main = 0;
	}

	int	i,n;
	for( i = 0 ; i < 61; i++ ){			/// Wait 16384 * 2000000/32768 Cycle
		for( n = 0 ; n < 16384; n++ ){
			asm ( "nop" );
		}
	}

	/// T8 Initialize
	rnf = InitT8( 0, 0xe, 0x79 );
	if( rnf == 0 ){
		printf("***  InitT8 ERROR  ***\n");
		rnf_main = 0;
	}
	/// T8 Interrupt Initialize
	rnf = InitT8Int( 3 );
	if( rnf == 0 ){
		printf("***  InitT8Int ERROR  ***\n");
		rnf_main = 0;
	}
	/// Interrupt enable
	EnableT8Int();
	/// T8 Timer Start
	StartT8();


	printf( "<<< Watchdog timer demonstration start >>>\n" );

	/// WDT Initialize
	rnf = InitWDT( 0 );
	if( rnf == 0 ){
		printf("***  InitWDT ERROR  ***\n");
		rnf_main = 0;
	}
	/// WDT Start
	StartWDT();

	T8LoopCount = 0;
	while( T8LoopCount < 10 ){
		asm ( "ei" );
		asm ( "halt" );
 		asm ( "nop" );
		asm ( "di" );

		/// Check T8 interrupt
		if ( t8Flag == 1 ){
			printf("*** T8 timer : reset watchdog timer ***\n");
			t8Flag = 0;
			T8LoopCount++;
		}
	}

	printf("*** stop T8 timer ***\n");
	/// Timer Stop
	StopT8();
	/// Interrupt Disable
	rnf = InitT8Int( 0 );
	if( rnf == 0 ) {
		printf("***  InitT8Int ERROR  ***\n");
		rnf_main = 0;
	}
	/// Interrupt disable
	DisableT8Int();

	/// Wait NMI interrupt
	while( 1 ){
		asm("halt");
		asm("nop");

		/// Check NMI interrupt
		if ( nmiFlag == 1 ){
			rnf = isWDTnmi();
			if( rnf == 1 ){
				printf( "*** NMI occurred ***\n" );
			} else {
				printf("*** NMI not occurred by WDT***\n");
			}
			break;
		}
		/// WDT clear
		ResetWDT();
	}

	/// WDT Stop
	StopWDT();

	printf( "<<< Watchdog timer demonstration finish >>>\n" );

	return ( rnf_main );
}

/*************************************************************************
 * IntT8 function.
 *
 * @brief	T8 interrupt routine.
 ************************************************************************/
void IntT8(void)
{
	if ( T8_INT0_T8IF == 1 ){
		/// Interrupt Flag Reset
		T8_INT0_T8IF = 1;
		/// WDT clear
		ResetWDT();
		t8Flag = 1;
	}
}

/****************************************************************************
 * nmi function.
 *
 * @brief	nmi interrupt routine.
 ***************************************************************************/
void nmi(void)
{
	/// Processing is undecided.
	nmiFlag = 1;
}

